/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: Dialog.cpp,v 2.10 2000/05/13 01:46:13 jeff Exp $

*******************************************************************************/


#include <iostream.h>

#include "Dialog.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#include "GenerateDialog.h"
#include "DlgParser.h"

#include "DialogWnd.h"
#include "ProjectWnd.h"

Dialog::Dialog( ProjectWnd &prj, const QString &name )
  : DlgObject( prj, name ),  
    itsWindowType( WT_Dialog ),
    isDialogModal( TRUE ),
    itsWindowFlags( static_cast<Qt::WidgetFlags>(0) )
{
  itsWindow = new DialogWnd( this, &prj, "" );
  if( ! itsWindow )
    fatal( tr( "Failed to new a Dialog..." ) );

  itsLayout = itsWindow->GetLayout();

  /* Create three popupmenus for the List:
   * 1. Not in a project
   * 2. In a project, not hidden
   * 3. In a project, hidden
   */
  unboundPopup = new QPopupMenu;
  unboundPopup->insertItem( tr( "Save" ), this, 
                              SLOT(Save()) );
  unboundPopup->insertItem( tr( "Save As..." ), this, 
                              SLOT(SaveAs()) );
  unboundPopup->insertItem( tr( "Close" ), this, 
                              SLOT(Hide()) );
  unboundPopup->insertItem( tr( "Delete" ), this, 
                              SLOT(Delete()) );
  unboundPopup->insertSeparator();
  unboundPopup->insertItem( tr( "Raise" ), this, 
                              SLOT(Raise()) );

  unboundHiddenPopup = new QPopupMenu;
  unboundHiddenPopup->insertItem( tr( "Show" ), this, SLOT(Show()) );

  boundPopup = new QPopupMenu;
  boundPopup->insertItem( tr( "Save" ), this, SLOT(Save()) );
  boundPopup->insertItem( tr( "Save As..." ), this, SLOT(SaveAs()) );
  boundPopup->insertSeparator();
  boundPopup->insertItem( tr( "Raise" ), this, SLOT(Raise()) );
  boundPopup->insertItem( tr( "Delete" ), this, 
                              SLOT(Delete()) );
  boundPopup->insertSeparator();
  boundPopup->insertItem( tr( "Remove" ), this, SLOT(Remove()) );

  // Popupmenu for closed dialog entries in projects
  boundHiddenPopup = new QPopupMenu;
  boundHiddenPopup->insertItem( tr( "Show" ), this, SLOT(Show()) );
  boundHiddenPopup->insertItem( tr( "Delete" ), this, 
                              SLOT(Delete()) );
  boundHiddenPopup->insertSeparator();
  boundHiddenPopup->insertItem( tr( "Remove" ), this, SLOT(Remove()) );

  status = DlgObject_Init;
  needsSave = FALSE;
}

Dialog::~Dialog()
{
  delete itsWindow;
}

void Dialog::GetGeneratedFiles( QString &src, 
                                QString &hdr, 
                                QString &mhdr )
{
  src = itsClassSource + " " + itsDataSource;
  hdr = itsClassHeader + " " + itsDataHeader;
  mhdr = hdr;
}

bool Dialog::Load()
{
  if( !itsFilename.isEmpty() )
  {
    if( fileTypeXML )
    {
      QtArchExpat parser( itsFilename );
      connect( &parser,
               SIGNAL(NewType(const XMLValue&)),
               SLOT(RestoreXML(const XMLValue&)) );
      parser.Parse();
    } else
    {
      itsParser = new DlgParser( itsFilename );

      connect( itsParser, SIGNAL(DlgKeyValue(QString &,QString &)), 
               SLOT(RestoreDialog(QString &,QString &)) );

      connect( itsParser, SIGNAL(NewLayout()), 
               SLOT(RestoreLayout()) );

      // And start the action
      itsParser->StartParsing();

      // That's it for the Parser object
      delete itsParser;    

      itsParser = NULL;
    }
    
    // HACK needed to get the window size right with layouts
    qApp->processEvents();
    itsLayout->InitialWindowResize();
  }

  status = DlgObject_Loaded;
  needsSave = FALSE;
  return TRUE;
}

void Dialog::RestoreXML( const XMLValue &value )
{
  if( value.GetTypeString() != "Dialog" )
    return;
  
  const XMLValueMap map = value.AsNested();

  const XMLValueMap common = map["DialogCommon"].AsNested();

  itsClassHeader = common["ClassHeader"].AsQString();
  itsClassSource = common["ClassSource"].AsQString();
  itsClassName = common["ClassName"].AsQString();
  itsDataHeader = common["DataHeader"].AsQString();
  itsDataSource = common["DataSource"].AsQString();
  itsDataName = common["DataName"].AsQString();
  itsDataVariableName = common["DataVariableName"].AsQString();
  QString wType = common["WindowBaseClass"].AsQString();
  if( wType == "QDialog" )
  {
    itsWindowType = WT_Dialog;
  }
  else if( wType == "QTabDialog" )
  {
    itsWindowType = WT_TabDialog;
  }
  else if( wType == "QWidget" )
  {
    itsWindowType = WT_Widget;
  }
  else if( wType == "QFrame" )
  {
    itsWindowType = WT_Frame;
  }
  else if( wType == "Custom" )
  {
    itsWindowType = WT_Custom;
  }
  SetIsDialogModal( common["IsModal"].AsBool());
  SetCaption( common["WindowCaption"].AsQString() );
  itsCustomBase = common["CustomBase"].AsQString();
  itsCustomBaseHeader = common["CustomBaseHeader"].AsQString();
  itsWindowFlags = static_cast<Qt::WidgetFlags>(common["WidgetFlags"].AsInt());
  itsLayout->RestoreXML( map["WidgetLayout"] );
}

void Dialog::RestoreDialog
(
 QString&	key,
 QString&	value
 )
{
  if( key == "ClassHeader" )
  {
    itsClassHeader = value;
  }
  else if( key == "ClassSource" )
  {
    itsClassSource = value;
  }
  else if( key == "ClassName" )
  {
    itsClassName = value;
  }
  else if( key == "DataHeader" )
  {
    itsDataHeader = value;
  }
  else if( key == "DataSource" )
  {
    itsDataSource = value;
  }
  else if( key == "DataName" )
  {
    itsDataName = value;
  }
  else if( key == "DataVariableName" )
  {
    itsDataVariableName = value;
  }
  else if( key == "WindowBaseClass" )
  {
    if( value == "QDialog" )
    {
	    itsWindowType = WT_Dialog;
    }
    else if( value == "QTabDialog" )
    {
	    itsWindowType = WT_TabDialog;
    }
    else if( value == "QWidget" )
    {
	    itsWindowType = WT_Widget;
    }
    else if( value == "QFrame" )
    {
	    itsWindowType = WT_Frame;
    }
    else if( value == "Custom" )
    {
	    itsWindowType = WT_Custom;
    }
  }
  else if( key == "IsModal" )
  {
    SetIsDialogModal( (value == "TRUE") );
  }
  else if( key == "WindowCaption" )
  {
    SetCaption( value );
  } 
  else if( key == "CustomBase" )
  {
    itsCustomBase = value;
  }
  else if( key == "CustomBaseHeader" )
  {
    itsCustomBaseHeader = value;
  }
  else if( key == "WindowFlags" )
  {
      itsWindowFlags = static_cast<Qt::WidgetFlags>(value.toInt());
  }
  else
  {
  }
}

void Dialog::RestoreLayout()
{
  itsLayout->RestoreContents( itsParser );
}

bool Dialog::Save( )
{
  QtArchGenerator generator;
  bool retVal = TRUE;
  
  try
  {
    generator.WriteStart( itsFilename );

    generator.WriteStartBlock( "QtArch", TRUE, 
                               "type=\"Dialog\" version=\"2.1\"" );

    generator.WriteStartBlock( "Dialog", TRUE, "supertype=\"true\"" );
    generator.WriteStartBlock( "DialogCommon" );
    generator.WriteSingleLine( "ClassHeader", itsClassHeader );
    generator.WriteSingleLine( "ClassSource", itsClassSource );
    generator.WriteSingleLine( "ClassName", itsClassName );
    generator.WriteSingleLine( "DataHeader", itsDataHeader );
    generator.WriteSingleLine( "DataSource", itsDataSource );
    generator.WriteSingleLine( "DataName", itsDataName );
    
    generator.WriteSingleLine( "WindowBaseClass", GetWindowStr() );
    generator.WriteSingleLine( "ClassHeader", itsClassHeader );
    
    if( (itsWindowType == WT_Dialog) || (itsWindowType == WT_TabDialog) )
    {
      generator.WriteXMLValue( "IsModal", XMLValue( isDialogModal ));
    }

    if( itsWindowType == WT_Custom )
    {
      generator.WriteSingleLine( "CustomBase", itsCustomBase );
      generator.WriteSingleLine( "CustomBaseHeader", itsCustomBaseHeader );    
    }

    if( !itsCaption.isEmpty() )
    {
      generator.WriteSingleLine( "WindowCaption", itsCaption );    
    }

    generator.WriteXMLValue( "WindowFlags", XMLValue( itsWindowFlags ) );    

    // End DialogCommon
    generator.WriteEndBlock();
    
    // Pump out the widgets
    itsLayout->Save( generator );
    
    // End Dialog
    generator.WriteEndBlock();

    // End QtArch
    generator.WriteEndBlock();
    generator.WriteEnd();

    status = DlgObject_Saved;
    needsSave = FALSE;
    emit Saved( this );

  }
  catch( GeneratorError &err )
  {
      QMessageBox::warning( 
          itsWindow, 
          "QtArchitect - Error",
          "Error saving " + itsFilename + ": " + err.message );

    retVal = FALSE;
  }

  return retVal;
}

bool Dialog::Generate( bool force )
{
    GenerateDialog dlg( itsFilename, itsClassName,
                        itsClassHeader, itsClassSource,
                        itsDataHeader, itsDataSource, itsWindow );

    if( force || dlg.exec() )
    {
        QFileInfo fileInfo( GetName() );
        QDir fileDir = fileInfo.dir();
      
        if( dlg.GenerateSource( itsClassHeader, itsClassSource ) )
        {
            GenerateSource( fileDir );
        }
	
        if( dlg.GenerateData( itsDataHeader, itsDataSource ) )
        {
	    GenerateData( fileDir );
        }	
    }

    status = DlgObject_Generated;
    return TRUE;
}

void Dialog::GenerateSource( const QDir& dir )
{
    if( itsClassHeader.isEmpty() )
    {
	QMessageBox::message( 
            tr( "Error" ), 
            tr( "Dialog class header file name not specified" ) );
	return;
    }
    
    if( itsClassSource.isEmpty() )
    {
	QMessageBox::message( 
            tr( "Error" ),
            tr( "Dialog class source file name not specified" ) );
	return;
    }

    if( itsDataHeader.isEmpty() )
    {
	QMessageBox::message( 
            tr( "Error" ), 
            tr( "Dialog data class header file name not specified" ) );
	return;
    }
    
    // generate header file
    QFile headerFile( QFileInfo( dir, itsClassHeader ).filePath() );
    if( !headerFile.open( IO_WriteOnly ) )
    {
	// error msg
	return;
    }
    
    QTextStream headerStream( &headerFile );
    
    OutputDlgeditHeader( headerStream, itsClassHeader, FALSE );
    GenerateSourceHeader( headerStream );
    
    headerFile.close();
    
  // generate source file
    QFile sourceFile( QFileInfo( dir, itsClassSource ).filePath() );
    if( !sourceFile.open( IO_WriteOnly ) )
    {
	// error msg
	return;
    }
    
    QTextStream sourceStream( &sourceFile );
    
    OutputDlgeditHeader( sourceStream, itsClassSource, FALSE );
    GenerateSourceSource( sourceStream );
    
    sourceFile.close();
}


void Dialog::GenerateSourceHeader
(
    QTextStream&	stream
)
{
    stream << "#ifndef " << itsClassName << "_included\n"
           << "#define " << itsClassName << "_included\n"
           << endl
           << "#include \"" << itsDataHeader << "\"\n"
           << endl
           << "class " << itsClassName << " : public " << itsDataName << '\n'
           << "{\n"
           << DlgWidget::srcIndent << "Q_OBJECT\n"
           << endl
           << "public:\n"
           << endl
           << DlgWidget::srcIndent << itsClassName << "(" << endl
           << DlgWidget::srcIndent << DlgWidget::srcIndent 
           << "QWidget *parent = NULL," << endl
           << DlgWidget::srcIndent << DlgWidget::srcIndent
           << "const char *name = NULL );\n"
           << endl
           << DlgWidget::srcIndent << "virtual ~" << itsClassName << "();\n";

    // Define slots...
    QDict<QString> publicSlots, protectedSlots;
      
    publicSlots.setAutoDelete( TRUE );
    protectedSlots.setAutoDelete( TRUE );
 
    QString baseClass;
    if( itsWindowType == WT_Custom )
    {
        baseClass = itsCustomBase;
    } else
    {
        baseClass = GetWindowStr();
    }
 
    itsLayout->GetAllSlots(&publicSlots, &protectedSlots, itsWindowType==WT_Custom ? itsCustomBase : GetWindowStr());

    QString* slotStr;

    if(publicSlots.count())
    {
        QDictIterator<QString> publicIter( publicSlots );

        stream << endl << DlgWidget::srcIndent << "public slots:\n\n";
        while( (slotStr = publicIter()) )
            stream << DlgWidget::srcIndent << "virtual void " << *slotStr << ";\n";
    }

    if(protectedSlots.count())
    {
        QDictIterator<QString> protectedIter( protectedSlots );
    
        stream << endl << DlgWidget::srcIndent << "protected slots:\n\n";
        while( (slotStr = protectedIter()) )
        {
            // if the user declared a slot to be both protected and private, make it public
            if( !publicSlots[ *slotStr ] )
                stream << DlgWidget::srcIndent << "virtual void " << *slotStr << ";\n";
        }
    }

    stream << "};\n"
           << "#endif // " << itsClassName << "_included\n";
}

void Dialog::GenerateSourceSource
(
    QTextStream&	stream
)
{
    stream << "#include \"" << itsClassHeader << "\"\n"

           << '\n'
           << "#define Inherited " << itsDataName << "\n"
           << '\n'
           << itsClassName << "::" << itsClassName << '\n'
           << "(\n"
           << "\tQWidget* parent,\n"
           << "\tconst char* name\n"
           << ")\n"
           << "\t:\n"
           << "\tInherited( parent, name )\n"
           << "{\n";
    
    if( !itsCaption.isEmpty() )
    {
        stream << DlgWidget::srcIndent << "setCaption( ";

        if( !itsCaption.isEmpty() && itsLayout->IsTextTranslated() )
        {
            stream << "tr( \"" << itsCaption << "\" )";
        }
        else
        {
            stream << "\"" << itsCaption << "\"";
        }
        
        stream << " );\n";
    }
    
    stream << "}\n"
           << endl
           << itsClassName << "::~" << itsClassName << "()\n"
           << "{\n"
           << "}\n";
    
    // define all the slots

    QDict<QString> publicSlots, protectedSlots;
      
    publicSlots.setAutoDelete( TRUE );
    protectedSlots.setAutoDelete( TRUE );
 
    itsLayout->GetAllSlots(&publicSlots, &protectedSlots, itsWindowType==WT_Custom ? itsCustomBase : GetWindowStr());

    QString* slotStr;

    if(publicSlots.count())
    {
        QDictIterator<QString> publicIter( publicSlots );

        while( (slotStr = publicIter()) )
            stream << "void " << itsClassName << "::" << *slotStr << "\n"
                   << "{\n"
                   << "}\n"
                   << "\n";
    }

    if(protectedSlots.count())
    {
        QDictIterator<QString> protectedIter( protectedSlots );
    
        while( (slotStr = protectedIter()) )
        {
            // if the user declared a slot to be both protected and private, make it public
            if( !publicSlots[ *slotStr ] )
                stream << "\nvoid " << itsClassName << "::" << *slotStr << "\n"
                       << "{\n"
                       << "}\n";
        }
    }
}


void Dialog::GenerateData( const QDir& dir )
{
    if( itsDataHeader.isEmpty() )
    {
	QMessageBox::message( 
            tr( "Error" ), 
            tr( "Dialog data header file name not specified" ) );
	return;
    }
    
    if( itsDataSource.isEmpty() )
    {
	QMessageBox::message( 
            tr( "Error" ), 
            tr( "Dialog data source file name not specified" ) );
	return;
    }
    
    // generate header file
    QFile headerFile( QFileInfo( dir, itsDataHeader ).filePath() );
    if( !headerFile.open( IO_WriteOnly ) )
    {
	// error msg
	return;
    }
    
    QTextStream headerStream( &headerFile );
    
    // Determine base class and header
    QString baseClass, baseClassHeader;
    if( itsWindowType == WT_Custom )
    {
	baseClass = itsCustomBase;
    } else
    {
	baseClass = GetWindowStr();
    }
    
    baseClassHeader = GetWindowHeader();
    
    OutputDlgeditHeader( headerStream, itsDataHeader, TRUE );
    itsLayout->GenerateDataHeader( headerStream, itsDataName,
				   baseClass, baseClassHeader );
    
    headerFile.close();
    
    // generate source file
    QFile sourceFile( QFileInfo( dir, itsDataSource ).filePath() );
    if( !sourceFile.open( IO_WriteOnly ) )
    {
	// error msg
	return;
    }
    
    QTextStream sourceStream( &sourceFile );
    
    bool isDialog = ( (itsWindowType == WT_Dialog) || 
		      (itsWindowType == WT_TabDialog) );

    OutputDlgeditHeader( sourceStream, itsDataSource, TRUE );
    itsLayout->GenerateDataSource( sourceStream, 
				   itsDataHeader, 	
				   itsDataName, 
				   baseClass,
				   isDialog,
				   isDialogModal,
				   itsWindowFlags );
    
    sourceFile.close();
}


void Dialog::OutputDlgeditHeader
(
    QTextStream&	stream,
    const QString&	filename,
    bool		editWarning
)
{
    stream << "/**********************************************************************\n\n";
    stream << "\t--- Qt Architect generated file ---\n\n";
    stream << "\tFile: " << filename << '\n';
    stream << "\tLast generated: " << 
	QDateTime::currentDateTime().toString() << '\n';
    stream << '\n';
    
    if( editWarning )
    {
	stream << "\tDO NOT EDIT!!!  This file will be automatically\n";
	stream << "\tregenerated by qtarch.  All changes will be lost.\n\n";
    }
    
    stream << " *********************************************************************/\n\n";
}


void Dialog::Hide()
{
  itsWindow->hide();
  status = DlgObject_Hidden;
}

void Dialog::Show()
{
  itsWindow->show();
  status = DlgObject_Shown;
}

void Dialog::Raise()
{
  itsWindow->raise();
}

void Dialog::Delete()
{
  // Leave all projects
  DlgObject::LeaveProject( 0 );

  // Announce our death
  emit Deleted( this );

  // I know that this is not absolutly correct, but it fit my needs
  delete this;
}

bool Dialog::EnterProject( const Project *prj )
{
  return DlgObject::EnterProject( prj );
}

bool Dialog::LeaveProject( Project *prj )
{
  return DlgObject::LeaveProject( prj );
}

void Dialog::CPAvailable( bool av )
{
    // propagate signal
    itsWindow->CPAvailable( av );
}

void Dialog::SetName( const QString &dlgFile, bool change )
{
  DlgObject::SetName( dlgFile, change );
  itsWindow->SetCaption( dlgFile );
}

const QString Dialog::GetWindowStr( void ) const
{
    QString str;
    
    switch( itsWindowType )
    {
	case WT_Dialog:
	{
	    str = "QDialog";
	    break;
	}
	case WT_TabDialog:
	{
	    str = "QTabDialog";
	    break;
	}
	case WT_Widget:
	{
	    str = "QWidget";
	    break;
	}
	case WT_Frame:
	{
	    str = "QFrame";
	    break;
	}
	case WT_Custom:
	{
	    str = "Custom";
	    break;
	}
    }
    
    return str;
}


const QString Dialog::GetWindowHeader( void ) const
{
    QString str;
    
    switch( itsWindowType )
    {
	case WT_Dialog:
	{
	    str = "<qdialog.h>";
	    break;
	}
	case WT_TabDialog:
	{
	    str = "<qtabdlg.h>";
	    break;
	}
	case WT_Widget:
	{
	    str = "<qwidget.h>";
	    break;
	}
	case WT_Frame:
	{
	    str = "<qframe.h>";
	    break;
	}
	case WT_Custom:
	{
	    str = "\"" + itsCustomBaseHeader + "\"";
	    break;
	}
    }
    
    return str;
}

void Dialog::CutSelectedWidgets()
{
  QtArchGenerator generator;
  QTextStream stream( &(itsProjectWnd.GetCPBuffer()), IO_WriteOnly );
  
  try
  {
    generator.WriteStart( &stream );

    generator.WriteStartBlock( "QtArch", TRUE, 
                               "type=\"Dialog\" version=\"2.1\"" );

    itsLayout->CutSelected( generator );
    
    generator.WriteEndBlock();
    generator.WriteEnd();
  }
  catch( GeneratorError &err )
  {
    debug( "Dialog cut failed: %s", (const char *) err.message );
  }

  emit CPDataSet( this );
  emit Changed( this );
}

void Dialog::CopySelectedWidgets()
{
  QtArchGenerator generator;
  QTextStream stream( &(itsProjectWnd.GetCPBuffer()), IO_WriteOnly );
  
  try
  {
    generator.WriteStart( &stream );

    generator.WriteStartBlock( "QtArch", TRUE, 
                               "type=\"Dialog\" version=\"2.1\"" );

    itsLayout->CopySelected( generator );
    
    generator.WriteEndBlock();
    generator.WriteEnd();
  }
  catch( GeneratorError &err )
  {
    debug( "Dialog cut failed: %s", (const char *) err.message );
  }

  emit CPDataSet( this );
}

void Dialog::PasteSelectedWidgets()
{
  QString fileName = itsProjectWnd.GetCPFileName();

  if( ! fileName.isEmpty() )
  {
    itsLayout->AutoSelect( TRUE );
    
    QtArchExpat parser( fileName );
    connect( &parser,
             SIGNAL(NewType(XMLValue&)),
             SLOT(RestoreXML(XMLValue&)) );
    parser.Parse();

    itsLayout->AutoSelect( FALSE );

    emit Changed( this );
  }
}

void Dialog::LayoutChanged()
{
  emit Changed( this );
  status = DlgObject_Changed;
  needsSave = TRUE;
}
