/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DialogWnd.cpp,v 2.14 2000/05/13 01:46:13 jeff Exp $

*******************************************************************************/


#include <iostream.h>

#include <qapp.h>
#include <qdatetm.h>
#include <qdir.h>
#include <qtstream.h>
#include <qfile.h>
#include <qfiledlg.h>
#include <qiconset.h>
#include <qkeycode.h>
#include <qpopmenu.h>
#include <qmsgbox.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>

#include "ProjectWnd.h"
#include "DialogWnd.h"
#include "GenerateDialog.h"
#include "DlgParser.h"
#include "WidgetModuleMgr.h"

#include "widget/DlgWidget.h"

#include "bmp/PixmapFactory.h"

#include "prop/DialogPropDialog.h"

#include "module/WidgetModule.h"

#define Inherited QMainWindow

DialogWnd::DialogWnd( Dialog* dialog,
                      ProjectWnd *project,
                      const char* name )
    : 
    Inherited( NULL, name ),
    itsKbdAccel( this ),
    itsStatusWidgetLabel( 0 ),
    itsStatusPosLabel( 0 ),
    itsStatusSizeLabel( 0 ),
    theProjectWnd( project ),
    itsDialog( dialog )
{
    setCaption( "Qt Architect" );

    setIcon( PixmapFactory::GetPixmap( QtArch_bmp ) );
    setIconText( caption() );

    setDockEnabled( QMainWindow::Top, TRUE );

    // create a new widget frame
    QWidget* centralWidget = new QWidget( this );

    itsActWidgetWnd = new WidgetFrame( centralWidget, "WidgetFrame" );
    itsActWidgetWnd->setFrameStyle( QFrame::Raised | QFrame::WinPanel );
    itsActWidgetWnd->setFixedSize( 400, 300 );
    itsActWidgetWnd->show();

    //
    // Setup the central widget
    //
    QHBoxLayout* leftRight = new QHBoxLayout( 5 );

    QVBoxLayout* topDown = new QVBoxLayout( centralWidget, 5 );
    topDown->addSpacing( 5 );
    topDown->addLayout( leftRight );
    topDown->addSpacing( 5 );
    topDown->addStretch( 1 );

    leftRight->addSpacing( 5 );
    leftRight->addWidget( itsActWidgetWnd, 0, AlignLeft | AlignTop );
    leftRight->addSpacing( 5 );
    leftRight->addStretch( 1 );

    setCentralWidget( centralWidget );

    // Create new layout
    itsActLayout = new WidgetLayout( this, name );

    itsActLayout->SetDlgEditWnd( itsActWidgetWnd );
    itsActLayout->SetNoMove( itsActWidgetWnd );
    itsActLayout->EnableLayout();

    // Dialog knows what to do when we want to close
    connect( this, SIGNAL(Closed()), dialog, SLOT(Hide()) );
  
    // Best for us if layout reports changes directly to dialog
    connect( itsActLayout, SIGNAL(Changed()), dialog, SLOT(LayoutChanged()) );

    connect( itsActLayout, SIGNAL(WidgetGeometryChanged(int,int,int,int)),
             SLOT(WidgetGeometryChanged(int,int,int,int)) );
    connect( itsActLayout, SIGNAL(WidgetSelected(const QString&,const QString&)),
             SLOT(WidgetSelected(const QString&, const QString&)) );
    connect( itsActLayout, SIGNAL(WidgetDeselected()),
             SLOT(WidgetDeselected()) );
    connect( itsActLayout, SIGNAL(UpdateScreen(int)), 
             SLOT(UpdateScreen(int)) );

    connect( itsActLayout, 
             SIGNAL(LayoutWndClosed()), 
             SLOT(LayoutWndClosed()) );

    // setup menu items
    QPopupMenu* file = new QPopupMenu;
    file->insertItem( PixmapFactory::GetPixmap( SaveDialogs_bmp ),
                      tr( "Save" ), 
                      dialog, 
                      SLOT(Save()), 
                      CTRL+Key_S );
    file->insertItem( PixmapFactory::GetPixmap( Generate_bmp ),
                      tr( "Generate Files" ), 
                      dialog, 
                      SLOT(Generate()), 
                      CTRL+Key_G );
    file->insertSeparator();
    file->insertItem( tr( "Close" ), this, SLOT(Close()), CTRL+Key_Q );

    QPopupMenu* align = new QPopupMenu;
    itsAlignTopMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( AlignTop_bmp ),
                           tr( "Align Top" ), 
                           itsActLayout, 
                           SLOT(AlignTop()),
                           ALT+Key_T );
    itsAlignBottomMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( AlignBottom_bmp ),
                           tr( "Align Bottom" ), 
                           itsActLayout, 
                           SLOT(AlignBottom()),
                           ALT+Key_B );
    itsAlignLeftMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( AlignLeft_bmp ),
                           tr( "Align Left" ), 
                           itsActLayout, 
                           SLOT(AlignLeft()),
                           ALT+Key_L );
    itsAlignRightMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( AlignRight_bmp ),
                           tr( "Align Right" ), 
                           itsActLayout, 
                           SLOT(AlignRight()),
                           ALT+Key_R );
    align->insertSeparator();
    itsAlignWidthMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( SizeHoriz_bmp ),
                           tr( "Same Width" ), 
                           itsActLayout, 
                           SLOT(SizeHorizontal()),
                           ALT+Key_W );
    itsAlignHeightMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( SizeVertical_bmp ),
                           tr( "Same Height" ), 
                           itsActLayout, 
                           SLOT(SizeVertical()),
                           ALT+Key_H );
    itsAlignSizeMenuId = 
        align->insertItem( PixmapFactory::GetPixmap( SizeHV_bmp ),
                           tr( "Same Size" ), 
                           itsActLayout, 
                           SLOT(SizeBothHV()),
                           ALT+Key_S );
  
    itsEditMenu = new QPopupMenu;
    cutMenuId = itsEditMenu->insertItem( PixmapFactory::GetPixmap( Cut_bmp ),
                                         tr( "Cut" ), 
                                         dialog, 
                                         SLOT(CutSelectedWidgets()),
                                         CTRL+Key_X );
    copyMenuId = itsEditMenu->insertItem( PixmapFactory::GetPixmap( Copy_bmp ),
                                          tr( "Copy" ), 
                                          dialog, 
                                          SLOT(CopySelectedWidgets()),
                                          CTRL+Key_C );
    pasteMenuId = itsEditMenu->insertItem( PixmapFactory::GetPixmap(Paste_bmp),
                                           tr( "Paste" ), 
                                           dialog, 
                                           SLOT(PasteSelectedWidgets()),
                                           CTRL+Key_V );
    itsEditDeleteMenuId = 
        itsEditMenu->insertItem( tr( "Delete" ), itsActLayout, 
                                 SLOT(DeleteSelectedWidget()), Key_Delete );
    itsEditMenu->insertSeparator();
    itsEditRaiseMenuId = 
        itsEditMenu->insertItem( tr( "Raise" ), itsActLayout, 
                                 SLOT(RaiseSelectedWidget()),
                                 CTRL+Key_R );
    itsEditLowerMenuId = 
        itsEditMenu->insertItem( tr( "Lower" ), itsActLayout, 
                                 SLOT(LowerSelectedWidget()),
                                 CTRL+Key_L );
    itsEditSizeToFitMenuId = 	
        itsEditMenu->insertItem( tr( "Size to Fit" ), itsActLayout,
                                 SLOT(SizeToFit()), CTRL+Key_F );
    itsEditMenu->insertItem( tr( "Properties..." ), itsActLayout, 
                             SLOT(SelectedWidgetProperties()));
    itsEditMenu->insertSeparator();
    itsAlignMenuId = itsEditMenu->insertItem( tr( "Align" ), align );
    itsEditMenu->insertSeparator();
    itsEditMenu->insertItem( tr( "Grid..." ), 
                             itsActLayout, 
                             SLOT(OptionsGrid()));

    itsEditMenu->setItemEnabled( cutMenuId, FALSE );
    itsEditMenu->setItemEnabled( copyMenuId, FALSE );
    itsEditMenu->setItemEnabled( pasteMenuId, FALSE );

    QPopupMenu* viewMenu = new QPopupMenu;
    viewMenu->setCheckable( TRUE );
    itsViewLayoutMgrMenuId = 
        viewMenu->insertItem( tr( "&Layout Manager" ), this, SLOT(ViewLayoutMgr()) );
    
    QPopupMenu* newWidget = new QPopupMenu;

    QPopupMenu *buttons = new QPopupMenu;
    newWidget->insertItem( tr( "Buttons" ), buttons );
    {
        buttons->insertItem( "Button", this, SLOT(NewButton()) );
        buttons->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_CheckBox ),
                             "Check Box", 
                             this, 
                             SLOT(NewCheckBox()),
                             CTRL+Key_2 );
        buttons->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_PushButton ),
                             "Push Button", 
                             this, 
                             SLOT(NewPushButton()), 
                             CTRL+Key_1 );
        buttons->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_RadioButton ),
                             "Radio Button", 
                             this, 
                             SLOT(NewRadioButton()),
                             CTRL+Key_3 );
    }     

    newWidget->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ComboBox ),
                           "Combo Box", 
                           this, 
                           SLOT(NewComboBox()),
                           CTRL+Key_9 );

    QPopupMenu *frames = new QPopupMenu;
    newWidget->insertItem( tr( "Frames" ), frames );
    {
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_Frame ),
                            "Frame", 
                            this, 
                            SLOT(NewFrame()),
                            ALT+Key_4 );
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_Grid ),
                            "Grid", 
                            this, 
                            SLOT(NewGrid()) );
    
        QPopupMenu *groupBoxes = new QPopupMenu;
        frames->insertItem( tr( "Group Boxes" ), groupBoxes );
        {
            groupBoxes->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_GroupBox ),
                                    "Group Box", 
                                    this, 
                                    SLOT(NewGroupBox()) );

            groupBoxes->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_HGroupBox ),
                                    "Horizontal Group Box",
                                    this,
                                    SLOT(NewHGroupBox()) );

            groupBoxes->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_VGroupBox ),
                                    "Vertical Group Box",
                                    this,
                                    SLOT(NewVGroupBox()) );

            groupBoxes->insertSeparator();

            groupBoxes->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ButtonGroup ),
                                    "Button Group", 
                                    this, 
                                    SLOT(NewButtonGroup()),
                                    CTRL+Key_4 );
            groupBoxes->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_HButtonGroup ),
                                    "Horizontal Button Group", 
                                    this, 
                                    SLOT(NewHButtonGroup()) );
            groupBoxes->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_VButtonGroup ),
                                    "Vertical Button Group", 
                                    this, 
                                    SLOT(NewVButtonGroup()) );
        }

        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_HBox ),
                            "Horizontal Box", 
                            this, 
                            SLOT(NewHBox()) );

        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_VBox ),
                            "Vertical Box", 
                            this, 
                            SLOT(NewVBox()) );

        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_LCDNumber ),
                            "LCD Number", 
                            this, 
                            SLOT(NewLCDNumber()),
                            CTRL+Key_6 );
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_Label ),
                            "Label", 
                            this, 
                            SLOT(NewLabel()),
                            CTRL+Key_5 );
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_MenuBar ),
                            "Menu Bar", 
                            this, 
                            SLOT(NewMenuBar()),
                            ALT+Key_5 );

        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ProgressBar ),
                            "Progress Bar", 
                            this, 
                            SLOT(NewProgressBar()),
                            ALT+Key_3 );

        QPopupMenu *scrollViews = new QPopupMenu;
        frames->insertItem( tr( "Scrollviews" ), scrollViews );
        {
            scrollViews->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ScrollView ),
                                     "Scroll View", 
                                     this, 
                                     SLOT(NewScrollView()),
                                     ALT+Key_9 );
            scrollViews->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_IconView ),
                                     "IconView", 
                                     this, 
                                     SLOT(NewIconView()));
            scrollViews->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ListBox ),
                                     "List Box", 
                                     this, 
                                     SLOT(NewListBox()),
                                     CTRL+Key_0 );
            scrollViews->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ListView ),
                                     "List View", 
                                     this, 
                                     SLOT(NewListView()),
                                     ALT+Key_0 );
            scrollViews->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_TextView ),
                                     "TextView", 
                                     this, 
                                     SLOT(NewTextView()) );
            scrollViews->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_TextBrowser ),
                                     "TextBrowser", 
                                     this, 
                                     SLOT(NewTextBrowser()));
        }
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_SpinBox ),
                            "Spin Box", 
                            this, 
                            SLOT(NewSpinBox()),
                            ALT+Key_7 );
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_Splitter ),
                            "Splitter", 
                            this, 
                            SLOT(NewSplitter()),
                            ALT+Key_8 );
        frames->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_MultiLineEdit ),
                            "Multi Line Edit", 
                            this, 
                            SLOT(NewMultiLineEdit()),
                            CTRL+Key_8 );
    }
    
    newWidget->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_LineEdit ),
                           "Line Edit", 
                           this, 
                           SLOT(NewLineEdit()),
                           CTRL+Key_7 );
    newWidget->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_ScrollBar ),
                           "Scroll Bar", 
                           this, 
                           SLOT(NewScrollBar()),
                           ALT+Key_1 );
    newWidget->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_Slider ),
                           "Slider", 
                           this, 
                           SLOT(NewSlider()),
                           ALT+Key_2 );
    newWidget->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_TabBar ),
                           "Tab Bar", 
                           this, 
                           SLOT(NewTabBar()),
                           ALT+Key_9 );
    newWidget->insertItem( DlgWidget::GetPixmap( DlgWidget::DW_User ),
                           "User Widget", 
                           this, 
                           SLOT(NewUser()),
                           ALT+Key_6 );

    QMenuBar& menu = *menuBar();
    menu.insertItem( tr( "&Dialog" ), file );
    menu.insertItem( tr( "&Edit" ), itsEditMenu );
    menu.insertItem( tr( "&View" ), viewMenu );
    menu.insertItem( tr( "&Insert" ), newWidget ); 


    //
    // Setup the edit toolbar
    //
    QToolBar* editToolBar = new QToolBar( tr( "Edit" ), this );

    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( SaveDialogs_bmp ),
                                      QIconSet::Small ),
                            tr( "Save Dialog (Ctrl+S)" ), 
                            tr( "Save the Dialog" ),
                            dialog,
                            SLOT(Save()),
                            editToolBar );

    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( Generate_bmp ),
                                      QIconSet::Small ),
                            tr( "Generate Dialog (Ctrl+G)" ), 
                            tr( "Generate the code for the Dialog" ),
                            dialog,
                            SLOT(Generate()),
                            editToolBar );

    editToolBar->addSeparator();

    itsCutTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( Cut_bmp ), 
                                   QIconSet::Small ),
                         tr( "Cut (Ctrl+X)" ),	
                         tr( "Cut Selected Widgets to Clipboard" ),
                         dialog,
                         SLOT(CutSelectedWidgets()),
                         editToolBar );
    itsCutTbId->setEnabled( FALSE );

    itsCopyTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( Copy_bmp ), 
                                   QIconSet::Small ),
                         tr( "Copy (Ctrl+C)" ),	
                         tr( "Copy Selected Widgets to Clipboard" ),
                         dialog,
                         SLOT(CopySelectedWidgets()),
                         editToolBar );
    itsCopyTbId->setEnabled( FALSE );

    itsPasteTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( Paste_bmp ), 
                                   QIconSet::Small ),
                         tr( "Paste (Ctrl+V)" ),	
                         tr( "Paste Widgets from Clipboard" ),
                         dialog,
                         SLOT(PasteSelectedWidgets()),
                         editToolBar );
    itsPasteTbId->setEnabled( FALSE );


    //
    // Setup the alignment toolbar
    //
    QToolBar* alignToolBar = new QToolBar( tr( "Align" ), this );
    
    itsAlignTopTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( AlignTop_bmp ), 
                                   QIconSet::Small ),
                         tr( "Align Top (Alt+T)" ), 
                         tr( "Align selected widgets on the top" ),
                         itsActLayout, 
                         SLOT(AlignTop()),
                         alignToolBar );

    itsAlignBottomTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( AlignBottom_bmp ),
                                   QIconSet::Small ),
                         tr( "Align Bottom (Alt+B)" ),
                         tr( "Align selected widgets on the bottom" ),
                         itsActLayout, 
                         SLOT(AlignBottom()),
                         alignToolBar );

    itsAlignLeftTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( AlignLeft_bmp ),
                                   QIconSet::Small ),
                         tr( "Align Left (Alt+L)" ),
                         tr( "Align selected widgets on the left" ),
                         itsActLayout, 
                         SLOT(AlignLeft()),
                         alignToolBar );

    itsAlignRightTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( AlignRight_bmp ),
                                   QIconSet::Small ),
                         tr( "Align Right (Alt+R)" ),
                         tr( "Align selected widgets on the right" ),
                         itsActLayout, 
                         SLOT(AlignRight()),
                         alignToolBar );

    alignToolBar->addSeparator();

    itsAlignWidthTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( SizeHoriz_bmp ),
                                   QIconSet::Small ),
                         tr( "Size Horizontally (Alt+H)" ),
                         tr( "Resize selected widgets horizontally" ),
                         itsActLayout, 
                         SLOT(SizeHorizontal()),
                         alignToolBar );

    itsAlignHeightTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( SizeVertical_bmp ),
                                   QIconSet::Small ),
                         tr( "Size Vertically (Alt+V)" ),
                         tr( "Resize selected widgets vertically" ),
                         itsActLayout, 
                         SLOT(SizeVertical()),
                         alignToolBar );

    itsAlignSizeTbId = 
        new QToolButton( QIconSet( PixmapFactory::GetPixmap( SizeHV_bmp ),
                                   QIconSet::Small ),
                         tr( "Size Horizontally and Vertically (Alt+S)" ),
                         tr( "Resize selected widgets horizontally and vertically" ),
                         itsActLayout, 
                         SLOT(SizeBothHV()),
                         alignToolBar );


    //
    // Setup widget toolbar
    //
    QToolBar* widgetToolBar = new QToolBar( "Widget", 
                                            this,
                                            QMainWindow::Top,
                                            TRUE /*newLine*/ );

    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_PushButton ),
                                      QIconSet::Small ),
                            tr( "New QPushButton (Ctrl+1)" ),
                            tr( "Create a new QPushButton" ),
                            this, 
                            SLOT(NewPushButton()),
                            widgetToolBar );

    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_CheckBox ),
                                      QIconSet::Small ),
                            tr( "New QCheckBox (Ctrl+2)" ), 
                            tr( "Create a new QCheckBox" ),
                            this, 
                            SLOT(NewCheckBox()), 
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_RadioButton ),
                                      QIconSet::Small ),
                            tr( "New QRadioButton (Ctrl+3)" ), 
                            tr( "Create a new QRadioButton" ),
                            this,
                            SLOT(NewRadioButton()),
                            widgetToolBar );
    
    (void)  new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ButtonGroup ),
                                       QIconSet::Small ),
                             tr( "New QButtonGroup (Ctrl+4)" ), 
                             tr( "Create a new QButtonGroup" ),
                             this, 
                             SLOT(NewButtonGroup()),
                             widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_Label ),
                                      QIconSet::Small ),
                            tr( "New QLabel (Ctrl+5)" ),
                            tr( "Create a new QLabel" ),
                            this, 
                            SLOT(NewLabel()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_LCDNumber ),
                                      QIconSet::Small ),
                            tr( "New QLCDNumber (Ctrl+6)" ),
                            tr( "Create a new QLCDNumber" ),
                            this, 
                            SLOT(NewLCDNumber()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_LineEdit ),
                                      QIconSet::Small ),
                            tr( "New QLineEdit (Ctrl+7)" ),
                            tr( "Create a new QLineEdit" ),
                            this, 
                            SLOT(NewLineEdit()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_MultiLineEdit ),
                                      QIconSet::Small ),
                            tr( "New QMultiLineEdit (Ctrl+8)" ), 
                            tr( "Create a new QMultiLineEdit" ),
                            this, 
                            SLOT(NewMultiLineEdit()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ComboBox ),
                                      QIconSet::Small ),
                            tr( "New QComboBox (Ctrl+9)" ),
                            tr( "Create a new QComboBox" ),
                            this,
                            SLOT(NewComboBox()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ListBox ),
                                      QIconSet::Small ),
                            tr( "New QListBox (Ctrl+0)" ),
                            tr( "Create a new QListBox" ),
                            this, 
                            SLOT(NewListBox()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ListView ),
                                      QIconSet::Small ),
                            tr( "New QListView (Alt+0)" ),
                            tr( "Create a new QListView" ),
                            this,
                            SLOT(NewListView()),
                            widgetToolBar );

    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_IconView ),
                                      QIconSet::Small ),
                            tr( "New QIconView" ),
                            tr( "Create a new QIconView" ),
                            this,
                            SLOT(NewIconView()),
                            widgetToolBar );

    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ScrollBar ),
                                      QIconSet::Small ),
                            tr( "New QScrollBar (Alt+1)" ), 
                            tr( "Create a new QScrollBar" ),
                            this, 
                            SLOT(NewScrollBar()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_Slider ),
                                      QIconSet::Small ),
                            tr( "New QSlider (Alt+2)" ),
                            tr( "Create a new QSlider" ),
                            this, 
                            SLOT(NewSlider()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_SpinBox ),
                                      QIconSet::Small ),
                            tr( "New QSpinBox (Alt+7)" ),
                            tr( "Create a new QSpinBox" ),
                            this,
                            SLOT(NewSpinBox()),
                            widgetToolBar );

    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_Splitter ),
                                      QIconSet::Small ),
                            tr( "New QSplitter (Alt+8)" ),
                            tr( "Create a new QSplitter" ),
                            this,
                            SLOT(NewSplitter()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ProgressBar ),
                                      QIconSet::Small ),
                            tr( "New QProgressBar (Alt+3)" ),
                            tr( "Create a new QProgressBar" ),
                            this,
                            SLOT(NewProgressBar()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_Frame ),
                                      QIconSet::Small ),
                            tr( "New QFrame (Alt+4)" ),
                            tr( "Create a new QFrame" ),
                            this, 
                            SLOT(NewFrame()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_ScrollView ),
                                      QIconSet::Small ),
                            tr( "New QScrollView (Alt+9)" ),
                            tr( "Create a new QScrollView" ),
                            this,
                            SLOT(NewScrollView()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_MenuBar ),
                                      QIconSet::Small ),
                            tr( "New QMenuBar (Alt+5)" ),
                            tr( "Create a new QMenuBar" ),
                            this,
                            SLOT(NewMenuBar()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_TabBar ),
                                      QIconSet::Small ),
                            tr( "New QTabBar (Alt+9)" ),
                            tr( "Create a new QTabBar" ),
                            this,
                            SLOT(NewTabBar()),
                            widgetToolBar );
    
    (void) new QToolButton( QIconSet( DlgWidget::GetPixmap( DlgWidget::DW_User ),
                                      QIconSet::Small ),
                            tr( "New User Widget (Alt+6)" ), 
                            tr( "Create a new user defineable widget" ),
                            this, 
                            SLOT(NewUser()),
                            widgetToolBar );


    //
    // Add any widget modules
    //

    QPopupMenu* moduleMenu = new QPopupMenu;
    const QList< WidgetModule >& widgetModules = WidgetModuleMgr::getModules();
    QListIterator< WidgetModule > moduleIter( widgetModules );
    WidgetModule* widgetModule = 0;
    short moduleNum = 0;
    bool firstModule = TRUE;

    while( (widgetModule = moduleIter()) )
    {
        QString moduleName = widgetModule->getName();

        QPopupMenu* moduleWidgetMenu = new QPopupMenu;
        connect( moduleWidgetMenu, 
                 SIGNAL(activated(int)), 
                 SLOT(NewModuleWidget(int)) );

        QToolBar* toolBar = 0;
        if( widgetModule->useToolbar() )
	{
            toolBar = new QToolBar( moduleName,
                                    this,
                                    QMainWindow::Top,
                                    firstModule /*newLine*/,
                                    moduleName );

            firstModule = FALSE;
	}

        QStrList widgetNames = widgetModule->getWidgetNames();
        QListIterator<char> widgetNameIter( widgetNames );
        const char* widgetName;
        short widgetNum = 0;
        static QString newText( tr( "New %1" ) );
        static QString createText( tr( "Create a new %1" ) );

        while( (widgetName = widgetNameIter()) )
	{
            QPixmap pixmap = widgetModule->getWidgetPixmap( widgetName );
	    
            if( !pixmap.isNull() )
	    {
                moduleWidgetMenu->insertItem( pixmap,
                                              widgetName,
                                              (moduleNum << 16 | widgetNum) );
		
                if( toolBar )
		{
                    (void) new QToolButton( QIconSet(pixmap, QIconSet::Small),
                                            newText.arg( widgetName ),
                                            createText.arg( widgetName ),
                                            this,
                                            SLOT(NewModuleWidget()),
                                            toolBar,
                                            widgetName );
		}
	    }
            else
	    {
                moduleWidgetMenu->insertItem( widgetName, 
                                              (moduleNum << 16 | widgetNum) );

                if( toolBar )
		{
                    QToolButton* toolButton = new QToolButton( toolBar,
                                                               widgetName );
                    toolButton->setUsesTextLabel( TRUE );
                    toolButton->setTextLabel( widgetName );
                    connect( toolButton, 
                             SIGNAL(clicked()), 
                             SLOT(NewModuleWidget()) );
		}
	    }

            widgetNum++;
	}
	
        moduleMenu->insertItem( moduleName, moduleWidgetMenu );

        moduleNum++;
    }

    if( moduleMenu->count() == 0 )
    {
        int id = moduleMenu->insertItem( tr( "No modules loaded" ) );
        moduleMenu->setItemEnabled( id, FALSE );
    }

    menu.insertItem( tr( "&Module" ), moduleMenu ); 

    //
    // Setup some accelerators
    //

    // Move widgets
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( Key_Up ),
			     itsActLayout, SLOT(MoveUp1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( Key_Down ),
			     itsActLayout, SLOT(MoveDown1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( Key_Left ),
			     itsActLayout, SLOT(MoveLeft1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( Key_Right ),
			     itsActLayout, SLOT(MoveRight1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT + Key_Up ),
			     itsActLayout, SLOT(MoveUpMultiple()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT + Key_Down ),
			     itsActLayout, SLOT(MoveDownMultiple()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT + Key_Left ),
			     itsActLayout, SLOT(MoveLeftMultiple()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT + Key_Right ),
			     itsActLayout, SLOT(MoveRightMultiple()) );
    // Resize widgets
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( SHIFT+Key_Up ),
			     itsActLayout, SLOT(ResizeShorter1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( SHIFT+Key_Down ),
			     itsActLayout, SLOT(ResizeTaller1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( SHIFT+Key_Left ),
			     itsActLayout, SLOT(ResizeNarrower1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( SHIFT+Key_Right ),
			     itsActLayout, SLOT(ResizeWider1()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT+SHIFT+Key_Up ),
			     itsActLayout, SLOT(ResizeShorterMultiple()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT+SHIFT+Key_Down ),
			     itsActLayout, SLOT(ResizeTallerMultiple()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT+SHIFT+Key_Left ),
			     itsActLayout, SLOT(ResizeNarrowerMultiple()) );
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( ALT+SHIFT+Key_Right ),
			     itsActLayout, SLOT(ResizeWiderMultiple()) );
    // Get widget properties
    itsKbdAccel.connectItem( itsKbdAccel.insertItem( Key_Return ),
			     itsActLayout, SLOT(SelectedWidgetProperties()) );


    //
    // Setup status bar
    //
    QStatusBar& statusbar = *statusBar();

    itsStatusWidgetLabel = new QLabel( &statusbar );
    statusbar.addWidget( itsStatusWidgetLabel, 3, FALSE );
    
    itsStatusPosLabel = new QLabel( &statusbar );
    statusbar.addWidget( itsStatusPosLabel, 1, TRUE );
    
    itsStatusSizeLabel = new QLabel( &statusbar );
    statusbar.addWidget( itsStatusSizeLabel, 1, TRUE );
    
    // Start snapping
    itsActWidgetWnd->SetGrid( 
	ProjectWnd::GetAppConfig().readEntry( "GridSize", (long int)10 ) 
	);

    itsActLayout->Start();

    //resize( 500, 450 );
}


DialogWnd::~DialogWnd()
{
    delete itsActLayout;
}

void DialogWnd::resizeEvent
(
    QResizeEvent*	//event
)
{
}


void DialogWnd::closeEvent
(
    QCloseEvent*	event
)
{
    // shut up
    event = event;

    QMenuBar& menu = *menuBar();
    menu.setItemChecked( itsViewLayoutMgrMenuId, FALSE );
    itsActLayout->ShowLayoutMgr( FALSE );

    emit Closed();
}


int DialogWnd::DoPropertyDialog( void )
{
    DialogPropDialog dlg( itsDialog );
    
    return dlg.exec();
}


void DialogWnd::NewButton()
{
    itsActLayout->AddWidget( DlgWidget::DW_Button );
}


void DialogWnd::NewButtonGroup()
{
    itsActLayout->AddWidget( DlgWidget::DW_ButtonGroup );
}


void DialogWnd::NewCheckBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_CheckBox );
}


void DialogWnd::NewComboBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_ComboBox );
}


void DialogWnd::NewFrame()
{
    itsActLayout->AddWidget( DlgWidget::DW_Frame );
}


void DialogWnd::NewGrid()
{
    itsActLayout->AddWidget( DlgWidget::DW_Grid );
}


void DialogWnd::NewGroupBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_GroupBox );
}


void DialogWnd::NewHBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_HBox );
}

void DialogWnd::NewHButtonGroup()
{
    itsActLayout->AddWidget( DlgWidget::DW_HButtonGroup );
}

void DialogWnd::NewHGroupBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_HGroupBox );
}

void DialogWnd::NewIconView()
{
    itsActLayout->AddWidget( DlgWidget::DW_IconView );
}


void DialogWnd::NewLCDNumber()
{
    itsActLayout->AddWidget( DlgWidget::DW_LCDNumber );
}


void DialogWnd::NewLabel()
{
    itsActLayout->AddWidget( DlgWidget::DW_Label );
}


void DialogWnd::NewLineEdit()
{
    itsActLayout->AddWidget( DlgWidget::DW_LineEdit );
}


void DialogWnd::NewListBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_ListBox );
}


void DialogWnd::NewListView()
{
    itsActLayout->AddWidget( DlgWidget::DW_ListView );
}


void DialogWnd::NewMenuBar()
{
    itsActLayout->AddWidget( DlgWidget::DW_MenuBar );
}


void DialogWnd::NewMultiLineEdit()
{
    itsActLayout->AddWidget( DlgWidget::DW_MultiLineEdit );
}


void DialogWnd::NewProgressBar()
{
    itsActLayout->AddWidget( DlgWidget::DW_ProgressBar );
}


void DialogWnd::NewPushButton()
{
    itsActLayout->AddWidget( DlgWidget::DW_PushButton );
}


void DialogWnd::NewRadioButton()
{
    itsActLayout->AddWidget( DlgWidget::DW_RadioButton );
}


void DialogWnd::NewScrollBar()
{
    itsActLayout->AddWidget( DlgWidget::DW_ScrollBar );
}


void DialogWnd::NewScrollView()
{
    itsActLayout->AddWidget( DlgWidget::DW_ScrollView );
}


void DialogWnd::NewSlider()
{
    itsActLayout->AddWidget( DlgWidget::DW_Slider );
}


void DialogWnd::NewSpinBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_SpinBox );
}

void DialogWnd::NewSplitter()
{
    itsActLayout->AddWidget( DlgWidget::DW_Splitter );
}


void DialogWnd::NewTabBar()
{
    itsActLayout->AddWidget( DlgWidget::DW_TabBar );
}


void DialogWnd::NewTextView()
{
    itsActLayout->AddWidget( DlgWidget::DW_TextView );
}

void DialogWnd::NewTextBrowser()
{
    itsActLayout->AddWidget( DlgWidget::DW_TextBrowser );
}


void DialogWnd::NewUser()
{
    itsActLayout->AddWidget( DlgWidget::DW_User );
}

void DialogWnd::NewVBox()
{
    itsActLayout->AddWidget( DlgWidget::DW_VBox );
}

void DialogWnd::NewVButtonGroup()
{
  itsActLayout->AddWidget( DlgWidget::DW_VButtonGroup );
}

void DialogWnd::NewVGroupBox()
{
  itsActLayout->AddWidget( DlgWidget::DW_VGroupBox );
}


void DialogWnd::NewModuleWidget( 
    int	theModuleId
    )
{
    int moduleNum = theModuleId >> 16;
    int moduleWidgetNum = theModuleId & 0xffff;

    itsActLayout->AddModuleWidget( moduleNum, moduleWidgetNum );
}


void DialogWnd::NewModuleWidget()
{
    const QObject* toolbarButton = sender();
    
    QString moduleWidgetName = toolbarButton->name();
    QString moduleName = toolbarButton->parent()->name();
    
    itsActLayout->AddModuleWidget( moduleName, moduleWidgetName );
}


void DialogWnd::ViewLayoutMgr()
{
    QMenuBar& menu = *menuBar();

    bool shown = menu.isItemChecked( itsViewLayoutMgrMenuId );
    shown = !shown;
    
    menu.setItemChecked( itsViewLayoutMgrMenuId, shown );
    itsActLayout->ShowLayoutMgr( shown );
}


void DialogWnd::LayoutWndClosed()
{
    menuBar()->setItemChecked( itsViewLayoutMgrMenuId, FALSE );
}


void DialogWnd::Close()
{
    itsActLayout->ShowLayoutMgr( FALSE );

    // Not really closing the dialog here. Masterclass Dialog will hide us 
    // instead
    emit Closed();
}


void DialogWnd::WidgetGeometryChanged( int x, int y,
				       int width, int height )
{
    QString str;

    str.sprintf( "X:%3d Y:%3d", x, y );
    itsStatusPosLabel->setText( str );

    str.sprintf( "W:%3d H:%3d", width, height );
    itsStatusSizeLabel->setText( str );
}


void DialogWnd::WidgetSelected( const QString&	widgetName,
				const QString&	varName )
{
    QString msg = widgetName.copy();

    if( !varName.isEmpty() )
    {
	msg += " (" + varName + ")";
    }

    itsStatusWidgetLabel->setText( msg );

    if( theProjectWnd )
    {
	itsEditMenu->setItemEnabled( cutMenuId, TRUE );
	itsEditMenu->setItemEnabled( copyMenuId, TRUE );

	itsCutTbId->setEnabled( TRUE );
	itsCopyTbId->setEnabled( TRUE );
    }
}

void DialogWnd::WidgetDeselected()
{
    itsStatusWidgetLabel->setText( "" );
    itsStatusPosLabel->setText( "" );
    itsStatusSizeLabel->setText( "" );

    itsEditMenu->setItemEnabled( cutMenuId, FALSE );
    itsEditMenu->setItemEnabled( copyMenuId, FALSE );

    itsCutTbId->setEnabled( FALSE );
    itsCopyTbId->setEnabled( FALSE );
}


void DialogWnd::CPAvailable( bool av )
{
    itsEditMenu->setItemEnabled( pasteMenuId, av );
    itsPasteTbId->setEnabled( av );
}


void DialogWnd::UpdateScreen( int flags )
{
    bool widgetSelected = FALSE;
    bool align = FALSE;

    if( flags & WidgetLayout::SF_WidgetSelected )
    {
	widgetSelected = TRUE;
    }
    
    if( flags & WidgetLayout::SF_Align )
    {
	align = TRUE;
    }

    QMenuBar& menu = *menuBar();
    menu.setItemEnabled( itsEditRaiseMenuId, widgetSelected );
    menu.setItemEnabled( itsEditLowerMenuId, widgetSelected );
    menu.setItemEnabled( itsEditDeleteMenuId, widgetSelected );
    menu.setItemEnabled( itsEditSizeToFitMenuId, widgetSelected );

    menu.setItemEnabled( itsAlignMenuId, align );
    menu.setItemEnabled( itsAlignTopMenuId, align );
    menu.setItemEnabled( itsAlignBottomMenuId, align );
    menu.setItemEnabled( itsAlignLeftMenuId, align );
    menu.setItemEnabled( itsAlignRightMenuId, align );
    menu.setItemEnabled( itsAlignWidthMenuId, align );
    menu.setItemEnabled( itsAlignHeightMenuId, align );
    menu.setItemEnabled( itsAlignSizeMenuId, align );

    itsAlignTopTbId->setEnabled( align );
    itsAlignBottomTbId->setEnabled( align );
    itsAlignLeftTbId->setEnabled( align );
    itsAlignRightTbId->setEnabled( align );
    itsAlignWidthTbId->setEnabled( align );
    itsAlignHeightTbId->setEnabled( align );
    itsAlignSizeTbId->setEnabled( align );
}

