/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgParser.cpp,v 2.4 1999/11/08 21:04:25 klaus Exp $

*******************************************************************************/


#include <qmsgbox.h>

#include "DlgParser.h"

DlgParser *gDlgParser = NULL;

extern FILE *yyin;

DlgParser::DlgParser( const QString &inFile, char *name )
    : QObject( NULL, name ),
      gridSizeSeen( FALSE )
{
  // Input stream OK ?
  if( ! (input = fopen( inFile, "r" ) ))
  {
    QString msg = tr( "Couldn't open %1 for reading" ).arg( inFile );
    QMessageBox::message( tr( "Error..." ), msg, tr( "OK" ) );
    initialized = FALSE;
    return;
  }

  // Read first line to retrieve version and type
  char line[256];
  fgets( line, 255, input );
  char *ptr = strtok( line, ":" );
  if( ! ptr )
  {
    // Old version, no information available
    QMessageBox::message( tr( "Error..." ), 
                          tr( "Please update this input file using DlgEditUpd" ),
                          tr( "OK" ) );
    initialized = FALSE;
    return;
  }
  if( strcmp( ptr, "DlgEdit" ) )
  {
    // Unknown version, no information available
    QMessageBox::message( tr( "Error..." ), 
                          tr( "Unknown header" ),
                          tr( "OK" ) );
    initialized = FALSE;
    return;
  }
  ptr = strtok( NULL, ":" );
  if( ! ptr )
  {
    // Unknown version, no information available
    QMessageBox::message( tr( "Error..." ), 
                          tr( "Header mismatch, No version or type information" ),
                          tr( "OK" ) );
    initialized = FALSE;
    return;
  }
  version = ptr;
  ptr = strtok( NULL, ":" );
  if( ! ptr )
  {
    // Unknown version, no information available
    QMessageBox::message( tr( "Error..." ), 
                          tr( "Header mismatch, No version or type information" ),
                          tr( "OK" ) );
    initialized = FALSE;
    return;
  }
  type = ptr;

  if( version != "v2.0" )
  {
    // Unknown version, no information available
    QMessageBox::message( "Error...", 
                          "Old version of dlg file. Please use update20.pl to update your dlg files",
                          "OK" );
    initialized = FALSE;
    return;
  }

  
  gDlgParser = this;

  yyin = input;

  // No Action
  action = FALSE;

  fileInfo.setFile( inFile );
  initialized = TRUE;
}

DlgParser::~DlgParser()
{
  if( input )
    fclose( input );
}

QString DlgParser::GetVersion()
{
  return version;
}

QString DlgParser::GetObjectType()
{
  return type;
}

void DlgParser::StartParsing( )
{
    // Action starting
    action = TRUE;

    // Parse until ready
    bool ready = FALSE;
  
    while( ! ready && action )
    {
	int retCode = yylex();
    
	switch( retCode )
	{
	    case EndOfInput: 
		ready = TRUE;
		continue;
	    case ProjectDataStart:
		emit NewProject();
		break;
	    case DialogDataStart:
		emit NewDialog();
		break;
	    case LayoutDataStart:
		emit NewLayout();
		break;
	    case WidgetDataStart:
		emit NewWidget( widgetType );
		break;
	    case LayoutItemDataStart:
		emit NewLayoutItem( widgetType );
		break;
	    case PixmapDataStart:
		emit NewPixmap();
		break;
	    case ProjectDataEnd:
		emit EndDialog();
		break;
	    case DialogDataEnd:
		emit EndDialog();
		break;
	    case WidgetDataEnd:
		emit EndWidget( );
		break;
	    case LayoutDataEnd:
		emit EndLayout( );
		break;
	    case LayoutItemDataEnd:
		emit EndLayoutItem();
		break;
	    case PixmapDataEnd:
		emit EndPixmap( );
		break;
	    case ProjectData:
		emit ProjectKeyValue( key, value );
		break;
	    case DialogData:
		emit DlgKeyValue( key, value );
		break;
	    case LayoutData:
		emit LayoutKeyValue( key, value );
		break;
	    case WidgetData:
		emit WidgetKeyValue( key, value );
		break;
	    case LayoutItemData:
		emit LayoutItemKeyValue( key, value );
		break;
	    case PixmapData:
		emit PixmapKeyValue( key, value );
		break;
	    case ErrorInInput:
		break;
	}

    }
}
void DlgParser::StopParsing()
{
  action = FALSE;
}
