/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridRow.cpp,v 2.4 2000/04/29 20:15:46 klaus Exp $

*******************************************************************************/


#include "layout/GridRow.h"
#include "layout/GridRowPropPage.h"
#include "layout/LayoutHelper.h"

#include "xml/QtArchGenerator.h"

#define Inherited LayoutItem

GridRow::GridRow(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_GridRow, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( GridRow_bmp ) ),
    myStretch( 1 ),
    mySpacing( 0 )
{
}


GridRow::GridRow(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_GridRow, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( GridRow_bmp ) ),
    myStretch( 1 ),
    mySpacing( 0 )
{
}


GridRow::~GridRow()
{
}


LayoutItemPropPage* GridRow::GetPropertyPage( QWidget*	parent,
					      bool	topLevel,
					      const WidgetLayout&	/*theWidgetLayout*/ )
{
    return new GridRowPropPage( this, topLevel, parent );
}


void GridRow::AddToLayout( QGridLayout*	theGridLayout,
			   int		theRowNum )
{
    // Add all the item in this row
    LayoutItem* item;
    int colNum;
    
    for( item = static_cast<LayoutItem*>(firstChild()), colNum = 0; 
	 item; 
	 item = static_cast<LayoutItem*>(item->nextSibling()), ++colNum )
    {
	item->AddToLayout( theGridLayout, theRowNum, colNum );
    }
    
    // Set the properties on this row
    theGridLayout->setRowStretch( theRowNum, myStretch );
    theGridLayout->addRowSpacing( theRowNum, mySpacing );
}


void GridRow::SaveContents( QtArchGenerator &generator ) const
{
  generator.WriteStartBlock( Type2String() );

  generator.WriteXMLValue( "Stretch", myStretch );
  generator.WriteXMLValue( "Spacing", mySpacing );

  SaveChildren( generator );

  generator.WriteEndBlock();
}


bool GridRow::RestoreKeyValue( const QString&	theKey,
			       const QString&	theValue,
			       const DlgWidget*	/*theDlgWidget*/ )
{
    bool rc = TRUE;

    if( theKey == "Stretch" )
    {
	myStretch = theValue.toInt();
    }
    else if( theKey == "Spacing" )
    {
	mySpacing = theValue.toInt();
    }
    else
    {
	rc = FALSE;
    }
    
    return rc;
}

void GridRow::Restore( const XMLValue &value, LayoutHelper *helper )
{
  // Sanity check
  if( value.GetTypeString() != Type2String() )
    return;

  const XMLValueMap map = value.AsNested();

  myStretch = map["Stretch"].AsInt();
  mySpacing = map["Spacing"].AsInt();

  RestoreChildren( map["Children"], helper );
}


void GridRow::GenerateSourceInGrid( QTextStream&	theStream,
				    const QString&	theLayoutVarName,
				    const QString&	theLayoutId,
				    unsigned int	theRowNum,
				    unsigned int	/*theColNum*/,
				    const QString&	theLayoutParentWidget ) const
{
    theStream << '\t' << theLayoutVarName << "->addRowSpacing( " <<
	theRowNum << ", " << mySpacing << " );\n";
    theStream << '\t' << theLayoutVarName << "->setRowStretch( " <<
	theRowNum << ", " << myStretch << " );\n";
    
    // Generate the children
    LayoutItem* item;
    QString idStr;
    unsigned int pos;

    for( item = static_cast<LayoutItem*>(firstChild()), pos = 1; 
	 item; 
	 item = static_cast<LayoutItem*>(item->nextSibling()), ++pos )
    {
	idStr = theLayoutId + '_' + idStr.setNum(pos);
	
	// Its children should be items in the grid.  The row num was passed
	// in.  The column number is assigned to it.
	item->GenerateSourceInGrid( theStream, 
				    theLayoutVarName,
				    idStr,
				    theRowNum,
				    pos - 1,
				    theLayoutParentWidget );
    }
}


QString GridRow::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Grid Row" );
    return treeLabel;
}
