/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridSpacer.h,v 2.4 2000/04/29 20:15:46 klaus Exp $

*******************************************************************************/


#ifndef GridSpacer_included
#define GridSpacer_included

#include <qlist.h>

#include "layout/LayoutItem.h"

class DlgMenuBar;
class QtArchGenerator;

class GridSpacer : public LayoutItem
{
public: // Methods

    GridSpacer(
	LayoutItem*	theParent
	);
    // Constructor

    GridSpacer(
	QListView*	theParent
	);
    // Constructor

    virtual ~GridSpacer();
    // Destructor

    virtual LayoutItemPropPage* GetPropertyPage( QWidget*	parent,
						 bool		topLevel,
						 const WidgetLayout&	theWidgetLayout );
    // Get the property page for the item.  All the information on the page 
    // should be updated.  The caller is responsible for freeing the
    // page when finished.

    //
    // Layout creation
    //

    virtual void AddToLayout( QGridLayout*	theGridLayout,
			      int		theRowNum,
			      int		theColNum );
    // Add the item and all its children to a grid layout.

    //
    // Loading and Saving
    //

    virtual void SaveContents( QtArchGenerator & ) const;
    // Save the contents of this layout item to theStream with theId as
    // its position identifier.
    
    virtual void Restore( const XMLValue &, LayoutHelper * );
    // Restore the item from the given XMLValue
    
    virtual bool RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	theDlgWidget );
    // Restore a key/value pair for the item.  theDlgWidget is the dialog
    // widget associated with theValue, if any.


    //
    // Code generation
    //

    virtual void GenerateSourceInGrid( QTextStream&	theStream,
				       const QString&	theLayoutVarName,
				       const QString&	theLayoutId,
				       unsigned int	theRowNum,
				       unsigned int	theColNum,
				       const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a grid layout.


private: // Class Methods

    static QString GetTreeLabel();
    // Get the label for the item in the tree
    
};


#endif // GridSpacer_included
