/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKKeyButton.cpp,v 2.2 1999/10/16 01:45:49 jeff Exp $

*******************************************************************************/


#include "DlgKKeyButton.h"

#include "KKeyButtonPropPage.h"

#include "GuiTools.h"

#define Inherited DlgPushButton

QStrList DlgKKeyButton::itsSignals;
int DlgKKeyButton::itsNextWidgetNum = 1;

DlgKKeyButton::DlgKKeyButton() :
    Inherited( DlgWidget::DW_PushButton )
{
}


DlgKKeyButton::DlgKKeyButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKKeyButton::~DlgKKeyButton()
{
}


void DlgKKeyButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KKeyButtonPropPage( this, parent, "KKeyButton" ) );

    labels.append( "KKeyButton" );
}


void DlgKKeyButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KKeyButton {" << endl;
    }
  
    KKeyButton* widget = GetKKeyButton();

    stream << saveIndent << "Editing {";
    SaveBool( stream, widget->editing );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKKeyButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KKeyButton* widget = GetKKeyButton();

    // Compare to known keys
    if( key == "Editing" )
    {
	widget->setEdit( (value == "TRUE") );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKKeyButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KKeyButton* widget = GetKKeyButton();

    stream << varName << "setEdit( " 
	   << (widget->editing ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgKKeyButton::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << '\t' << varName << " = new " << GetWidgetClass() << "( "
	   << name << ", " << parentName << " );" << endl;
}


void DlgKKeyButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKKeyButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKKeyButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KKeyButton* widget = 
	new KKeyButton( CreateName( "KKeyButton", 
				    theName, 
				    itsNextWidgetNum ),
			theParent );

    SetQtWidget( widget, theParent );
}
