/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKLed.cpp,v 2.2 1999/10/16 01:45:51 jeff Exp $

*******************************************************************************/


#include "DlgKLed.h"

#include "KLedPropPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgKLed::itsSignals;
int DlgKLed::itsNextWidgetNum = 1;

DlgKLed::DlgKLed() :
    Inherited( DlgWidget::DW_User )
{
}


DlgKLed::DlgKLed(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKLed::~DlgKLed()
{
}


void DlgKLed::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KLedPropPage( this, parent, "KLed" ) );

    labels.append( "KLed" );
}


void DlgKLed::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KLed {" << endl;
    }
  
    KLed* widget = GetKLed();

    stream << saveIndent << "State {";
    switch( widget->getState() )
    {
	case KLed::On:
	{
	    stream << "On";
	    break;
	}
	case KLed::Off:
	{
	    stream << "Off";
	    break;
	}
    }
    stream << "}" << endl;
    
    stream << saveIndent << "Color {";
    switch( widget->getColor() )
    {
	case KLed::yellow:
	{
	    stream << "yellow";
	    break;
	}
	case KLed::orange:
	{
	    stream << "orange";
	    break;
	}
	case KLed::red:
	{
	    stream << "red";
	    break;
	}
	case KLed::green:
	{
	    stream << "green";
	    break;
	}
	case KLed::blue:
	{
	    stream << "blue";
	    break;
	}
    }
    stream << "}" << endl;
    
    stream << saveIndent << "Look {";
    switch( widget->getLook() )
    {
	case KLed::flat:
	{
	    stream << "flat";
	    break;
	}
	case KLed::round:
	{
	    stream << "round";
	    break;
	}
	case KLed::sunken:
	{
	    stream << "sunken";
	    break;
	}
    }
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKLed::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KLed* widget = GetKLed();

    // Compare to known keys
    if( key == "State" )
    {
	KLed::State state = KLed::On;
	if( value == "On" )
	{
	    state = KLed::On;
	}
	else if( value == "Off" )
	{
	    state = KLed::Off;
	}
	
	widget->setState( state );
    } 
    else if( key == "Color" )
    {
	KLed::Color color = KLed::yellow;
	if( value == "yellow" )
	{
	    color = KLed::yellow;
	}
	else if( value == "orange" )
	{
	    color = KLed::orange;
	}
	else if( value == "red" )
	{
	    color = KLed::red;
	}
	else if( value == "green" )
	{
	    color = KLed::green;
	}
	else if( value == "blue" )
	{
	    color = KLed::blue;
	}
	
	widget->setColor( color );
    }
    else if( key == "Look" )
    {
	KLed::Look look = KLed::flat;
	if( value == "flat" )
	{
	    look = KLed::flat;
	}
	else if( value == "round" )
	{
	    look = KLed::round;
	}
	else if( value == "sunken" )
	{
	    look = KLed::sunken;
	}
	
	widget->setLook( look );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKLed::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KLed* widget = GetKLed();

    stream << varName << "setState( KLed::";
    switch( widget->getState() )
    {
	case KLed::On:
	{
	    stream << "On";
	    break;
	}
	case KLed::Off:
	{
	    stream << "Off";
	    break;
	}
    }
    stream << " );" << endl;
    
    stream << varName << "setLook( KLed::";
    switch( widget->getLook() )
    {
	case KLed::flat:
	{
	    stream << "flat";
	    break;
	}
	case KLed::round:
	{
	    stream << "round";
	    break;
	}
	case KLed::sunken:
	{
	    stream << "sunken";
	    break;
	}
    }
    stream << " );" << endl;
}


void DlgKLed::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << '\t' << varName << " = new " << GetWidgetClass() << "( ";

    stream << "KLed::";
    switch( GetKLed()->getColor() )
    {
	case KLed::yellow:
	{
	    stream << "yellow";
	    break;
	}
	case KLed::orange:
	{
	    stream << "orange";
	    break;
	}
	case KLed::red:
	{
	    stream << "red";
	    break;
	}
	case KLed::green:
	{
	    stream << "green";
	    break;
	}
	case KLed::blue:
	{
	    stream << "blue";
	    break;
	}
    }

    stream << ", " << parentName << ", " << name << " );" << endl;
}


void DlgKLed::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKLed::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKLed::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KLed* widget = 
	new KLed( KLed::yellow,
		  theParent,
		  CreateName( "KLed", 
			      theName, 
			      itsNextWidgetNum ) );
    
    SetQtWidget( widget, theParent );
}
