/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKLined.cpp,v 2.2 1999/10/16 01:45:56 jeff Exp $

*******************************************************************************/


#include "DlgKLined.h"

#include "GuiTools.h"

#define Inherited DlgLineEdit

QStrList DlgKLined::itsSignals;
int DlgKLined::itsNextWidgetNum = 1;

DlgKLined::DlgKLined() :
    Inherited( DlgWidget::DW_LineEdit )
{
}


DlgKLined::DlgKLined(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKLined::~DlgKLined()
{
}


void DlgKLined::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );
}


void DlgKLined::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KLined {" << endl;
    }
  
    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKLined::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    retVal = Inherited::RestoreKeyValue( key, value, FALSE );

    return retVal;
}


void DlgKLined::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );
}


void DlgKLined::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKLined::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "completion()" );
	itsSignals.append( "rotation()" );
    }

    return itsSignals;
}


void DlgKLined::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KLined* widget = 
	new KLined( theParent,
		    CreateName( "KLined", 
				theName, 
				itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
