/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKListSpinBox.cpp,v 2.2 1999/10/16 01:45:58 jeff Exp $

*******************************************************************************/


#include "DlgKListSpinBox.h"

#include "KListSpinBoxPropPage.h"

#include "GuiTools.h"

#define Inherited DlgKSpinBox

QStrList DlgKListSpinBox::itsSignals;
int DlgKListSpinBox::itsNextWidgetNum = 1;

DlgKListSpinBox::DlgKListSpinBox() :
    Inherited( DlgWidget::DW_User ),
    myItems( TRUE /*deepCopy*/ )
{
}


DlgKListSpinBox::DlgKListSpinBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myItems( TRUE /*deepCopy*/ )
{
}


DlgKListSpinBox::~DlgKListSpinBox()
{
}


void DlgKListSpinBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KListSpinBoxPropPage( this, parent, "KListSpinBox" ) );

    labels.append( "KListSpinBox" );
}


void DlgKListSpinBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KListSpinBox {" << endl;
    }
  
    KListSpinBox* widget = GetKListSpinBox();

    QListIterator<char> iter( myItems );
    const char* item = 0;
    while( (item = iter()) )
    {
	stream << saveIndent << "Item {" << item << "}" << endl;
    }

    stream << saveIndent << "Index {" << widget->getIndex() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKListSpinBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KListSpinBox* widget = GetKListSpinBox();

    // Compare to known keys
    if( key == "Item" )
    {
	myItems.append( value );
    }
    else if( key == "Index" )
    {
	widget->setIndex( value.toUInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKListSpinBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KListSpinBox* widget = GetKListSpinBox();

    stream << varName << "setIndex( " << widget->getIndex() << " );" << endl;
}


void DlgKListSpinBox::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << "\t{" << endl;
    stream << "\t\tQStrList* items = new QStrList();" << endl;
    
    QListIterator<char> iter( myItems );
    const char* item = 0;
    while( (item = iter()) )
    {
	stream << "\t\titems->append( \"" << item << "\" );" << endl;
    }

    stream << "\t\t" << varName << " = new " << GetWidgetClass() << "( "
	   << "items, " << parentName << ", " << name << " );" << endl;

    stream << "\t}" << endl;
}


void DlgKListSpinBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKListSpinBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKListSpinBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KListSpinBox* widget = 
	new KListSpinBox( new QStrList(),
			  theParent,
			  CreateName( "KListSpinBox", 
				      theName, 
				      itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
