/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKNumCheckButton.cpp,v 2.2 1999/10/16 01:46:00 jeff Exp $

*******************************************************************************/


#include "DlgKNumCheckButton.h"

#include "KNumCheckButtonPropPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgKNumCheckButton::itsSignals;
int DlgKNumCheckButton::itsNextWidgetNum = 1;

DlgKNumCheckButton::DlgKNumCheckButton() :
    Inherited( DlgWidget::DW_User )
{
}


DlgKNumCheckButton::DlgKNumCheckButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKNumCheckButton::~DlgKNumCheckButton()
{
}


void DlgKNumCheckButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KNumCheckButtonPropPage( this, parent, "KNumCheckButton" ) );

    labels.append( "KNumCheckButton" );
}


void DlgKNumCheckButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KNumCheckButton {" << endl;
    }
  
    stream << saveIndent << "Text {" << myText << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKNumCheckButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Text" )
    {
	SetText( value );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKNumCheckButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    stream << varName << "setText( \"" << myText << "\" );" << endl;
}


void DlgKNumCheckButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKNumCheckButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "selected()" );
	itsSignals.append( "deselected()" );
	itsSignals.append( "doubleclick(bool)" );
    }

    return itsSignals;
}


void DlgKNumCheckButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KNumCheckButton* widget = 
	new KNumCheckButton( theParent,
			     CreateName( "KNumCheckButton", 
					 theName, 
					 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
