/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKSelector.cpp,v 2.2 1999/10/16 01:46:11 jeff Exp $

*******************************************************************************/


#include "DlgKSelector.h"

#include "KSelectorPropPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgKSelector::itsSignals;
int DlgKSelector::itsNextWidgetNum = 1;

DlgKSelector::DlgKSelector() :
    Inherited( DlgWidget::DW_User ),
    myOrientation( KSelector::Horizontal )
{
}


DlgKSelector::DlgKSelector(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myOrientation( KSelector::Horizontal )
{
}


DlgKSelector::~DlgKSelector()
{
}


void DlgKSelector::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KSelectorPropPage( this, parent, "KSelector" ) );

    labels.append( "KSelector" );
}


void DlgKSelector::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KSelector {" << endl;
    }
  
    KSelector* widget = GetKSelector();

    stream << saveIndent << "Orientation {";
    switch( myOrientation )
    {
	case KSelector::Horizontal:
	{
	    stream << "Horizontal";
	    break;
	}
	case KSelector::Vertical:
	{
	    stream << "Vertical";
	    break;
	}
    }
    stream << "}" << endl;

    stream << saveIndent << "Indent {";
    SaveBool( stream, widget->indent() );
    stream << "}" << endl;

    stream << saveIndent << "MinValue {" << widget->minValue() << "}" << endl;
    stream << saveIndent << "MaxValue {" << widget->maxValue() << "}" << endl;
    stream << saveIndent << "Initial {" << widget->value() << "}" << endl;
    stream << saveIndent << "LineStep {" << widget->lineStep() << "}" << endl;
    stream << saveIndent << "PageStep {" << widget->pageStep() << "}" << endl;
    
    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKSelector::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KSelector* widget = GetKSelector();

    // Compare to known keys
    if( key == "Orientation" )
    {
	if( value == "Horizontal" )
	{
	    myOrientation = KSelector::Horizontal;
	}
	else
	{
	    myOrientation = KSelector::Vertical;
	}
    } 
    else if( key == "Indent" )
    {
	widget->setIndent( (value == "TRUE") );
    }
    else if( key == "MinValue" )
    {
	widget->setRange( value.toInt(), widget->maxValue() );
    }
    else if( key == "MaxValue" )
    {
	widget->setRange( widget->minValue(), value.toInt() );
    }
    else if( key == "Initial" )
    {
	widget->setValue( value.toInt() );
    }
    else if( key == "LineStep" )
    {
	widget->setSteps( value.toInt(), widget->pageStep() );
    }
    else if( key == "PageStep" )
    {
	widget->setSteps( widget->lineStep(), value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKSelector::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KSelector* widget = GetKSelector();

    stream << varName << "setIndent( "
	   << (widget->indent() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setRange( " <<
	widget->minValue() << ", " << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( " <<
	widget->lineStep() << ", " << widget->pageStep() << " );" << endl;
    
    stream << varName << 
	"setValue( " << widget->value() << " );" << endl;
}


void DlgKSelector::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << '\t' << varName << " = new " << GetWidgetClass() 
	   << "( KSelector::";

    switch( myOrientation )
    {
	case KSelector::Horizontal:
	{
	    stream << "Horizontal";
	    break;
	}
	case KSelector::Vertical:
	{
	    stream << "Vertical";
	    break;
	}
    }

    stream << ", " << parentName << ", " << name << " );\n";
}


void DlgKSelector::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKSelector::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
    }

    return itsSignals;
}


void DlgKSelector::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KSelector* widget = 
	new KSelector( KSelector::Horizontal,
		       theParent,
		       CreateName( "KSelector", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
