/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKSlider.cpp,v 2.2 1999/10/16 01:46:15 jeff Exp $

*******************************************************************************/


#include "DlgKSlider.h"

#include "GuiTools.h"

#define Inherited DlgSlider

QStrList DlgKSlider::itsSignals;
int DlgKSlider::itsNextWidgetNum = 1;

DlgKSlider::DlgKSlider() :
    Inherited( DlgWidget::DW_Slider )
{
}


DlgKSlider::DlgKSlider(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKSlider::~DlgKSlider()
{
}


void DlgKSlider::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );
}


void DlgKSlider::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KSlider {" << endl;
    }
  
    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKSlider::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    retVal = Inherited::RestoreKeyValue( key, value, FALSE );

    return retVal;
}


void DlgKSlider::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );
}


void DlgKSlider::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKSlider::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKSlider::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KSlider* widget = 
	new KSlider( theParent,
		     CreateName( "KSlider", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
