/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKURLLabel.cpp,v 2.2 1999/10/16 01:46:24 jeff Exp $

*******************************************************************************/


#include <qapp.h>

#include "DlgKURLLabel.h"

#ifndef MODULE_BOOTSTRAP
#include "KURLLabelPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgLabel

QStrList DlgKURLLabel::itsSignals;
int DlgKURLLabel::itsNextWidgetNum = 1;

DlgKURLLabel::DlgKURLLabel() :
    Inherited( DlgWidget::DW_Label ),
    myIsGlow( TRUE ),
    myIsFloat( FALSE ),
    myIsUnderline( TRUE ),
    myTextAlignment( Left ),
    myHighlightedColor( blue ),
    mySelectedColor( red ),
    myBackgroundColor( QApplication::palette()->normal().background() )
{
}


DlgKURLLabel::DlgKURLLabel(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myIsGlow( TRUE ),
    myIsFloat( FALSE ),
    myIsUnderline( TRUE ),
    myTextAlignment( Left ),
    myHighlightedColor( blue ),
    mySelectedColor( red ),
    myBackgroundColor( QApplication::palette()->normal().background() )
{
}


DlgKURLLabel::~DlgKURLLabel()
{
}


void DlgKURLLabel::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgFrame::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KURLLabelPropPage( this, parent, "KURLLabel" ) );

    labels.append( "KURLLabel" );
#endif // MODULE_BOOTSTRAP
}


void DlgKURLLabel::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KURLLabel {" << endl;
    }
  
    KURLLabel* widget = GetKURLLabel();

    stream << saveIndent << "URLLabelText {" << widget->text() << "}" << endl;
    stream << saveIndent << "URL {" << myURL << "}" << endl;
    
    stream << saveIndent << "Glow {";
    SaveBool( stream, myIsGlow );
    stream << "}" << endl;
    
    stream << saveIndent << "Float {";
    SaveBool( stream, myIsFloat );
    stream << "}" << endl;
    
    stream << saveIndent << "Underline {";
    SaveBool( stream, myIsUnderline );
    stream << "}" << endl;
    
    stream << saveIndent << "TextAlignment {";
    switch( myTextAlignment )
    {
	case Top:
	{
	    stream << "Top";
	    break;
	}
	case Bottom:
	{
	    stream << "Bottom";
	    break;
	}
	case Left:
	{
	    stream << "Left";
	    break;
	}
	case Right:
	{
	    stream << "Right";
	    break;
	}
    }
    stream << "}" << endl;
    
    stream << saveIndent << "HighlightedColor {" 
	   << myHighlightedColor.rgb() << "}" << endl;

    stream << saveIndent << "SelectedColor {" 
	   << mySelectedColor.rgb() << "}" << endl;

    stream << saveIndent << "BackgroundColor {" 
	   << myBackgroundColor.rgb() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKURLLabel::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KURLLabel* widget = GetKURLLabel();

    // Compare to known keys
    if( key == "URLLabelText" )
    {
	widget->setText( value );
    }
    else if( key == "URL" )
    {
	setURL( value );
    }
    else if( key == "Glow" )
    {
	setGlow( (value == "TRUE" ) );
    }
    else if( key == "Float" )
    {
	setFloat( (value == "TRUE" ) );
    }
    else if( key == "Underline" )
    {
	setUnderline( (value == "TRUE" ) );
    }
    else if( key == "TextAlignment" )
    {
	TextAlignment align = Top;
	
	if( value == "Top" )
	{
	    align = Top;
	}
	else if( value == "Bottom" )
	{
	    align = Bottom;
	}
	else if( value == "Left" )
	{
	    align = Left;
	}
	else if( value == "Right" )
	{
	    align = Right;
	}
	
	setTextAlignment( align );
    }
    else if( key == "HighlightedColor" )
    {
	setHighlightedColor( QRgb( value.toUInt() ) );
    }
    else if( key == "SelectedColor" )
    {
	setSelectedColor( QRgb( value.toUInt() ) );
    }
    else if( key == "BackgroundColor" )
    {
	setBackgroundColor( QRgb( value.toUInt() ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKURLLabel::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KURLLabel* widget = GetKURLLabel();

    stream << varName << "setText( \"" << widget->text() << "\" );" << endl;
    stream << varName << "setURL( \"" << myURL << "\" );" << endl;
    
    stream << varName << "setGlow( "
	   << (myIsGlow ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setFloat( "
	   << (myIsFloat ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setUnderline( "
	   << (myIsUnderline ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setTextAlignment( ";
    switch( myTextAlignment )
    {
	case Top:
	{
	    stream << "Top";
	    break;
	}
	case Bottom:
	{
	    stream << "Bottom";
	    break;
	}
	case Left:
	{
	    stream << "Left";
	    break;
	}
	case Right:
	{
	    stream << "Right";
	    break;
	}
    }
    stream << " );" << endl;
    
    stream << varName << "setHighlightedColor( QColor( QRgb( "
	   << myHighlightedColor.rgb() << " ) ) );" << endl;

    stream << varName << "setSelectedColor( QColor( QRgb( "
	   << mySelectedColor.rgb() << " ) ) );" << endl;

    stream << varName << "setBackgroundColor( QColor( QRgb( "
	   << myBackgroundColor.rgb() << " ) ) );" << endl;

}


void DlgKURLLabel::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKURLLabel::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "enteredURL(const char*)" );
	itsSignals.append( "enteredURL()" );
	itsSignals.append( "leftURL(const char*)" );
	itsSignals.append( "leftURL()" );
	itsSignals.append( "leftClickedURL(const char*)" );
	itsSignals.append( "leftClickedURL()" );
	itsSignals.append( "rightClickedURL(const char*)" );
	itsSignals.append( "rightClickedURL()" );
	itsSignals.append( "middleClickedURL(const char*)" );
	itsSignals.append( "middleClickedURL()" );
    }

    return itsSignals;
}


void DlgKURLLabel::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KURLLabel* widget = 
	new KURLLabel( theParent,
		       CreateName( "KURLLabel", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
