/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgClass.cpp,v 2.1 1999/10/16 01:47:38 jeff Exp $

*******************************************************************************/


#include "DlgCCC.h"

#include "CCCPropPage.h"

#include "GuiTools.h"

#define Inherited DDD

QStrList DlgCCC::itsSignals;
int DlgCCC::itsNextWidgetNum = 1;

DlgCCC::DlgCCC() :
    Inherited( DlgWidget::DW_??? )
{
}


DlgCCC::DlgCCC(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgCCC::~DlgCCC()
{
}


void DlgCCC::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new CCCPropPage( this, parent, "CCC" ) );

    labels.append( "CCC" );
}


void DlgCCC::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::CCC {" << endl;
    }
  
    CCC* widget = GetCCC();

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgCCC::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    CCC* widget = GetCCC();

    // Compare to known keys
    if( key == "Color" )
    {
	widget->setColor( QColor( value.toUInt() ) );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgCCC::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const CCC* widget = GetCCC();

    stream << varName << "setColor( QColor( QRgb( " << widget->color().rgb()
	   << " ) ) );" << endl;
}


void DlgCCC::GenerateExtraSource( 
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateExtraSource( stream, varName, zOrdering );
}


void DlgCCC::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgCCC::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "changed(const QColor&)" );
    }

    return itsSignals;
}


void DlgCCC::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    CCC* widget = 
	new CCC( theParent,
			  CreateName( "CCC", 
				      theName, 
				      itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
