/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: PixmapWnd.h,v 2.1 1999/10/16 01:48:06 jeff Exp $

*******************************************************************************/


/*
 * PixmapWnd is a subclass of QFrame. It contains the following widgets: 
 * - Menubar
 * - Pixmap Toolbar
 * - Pixmap Previewer (only for pixmap up to 32x32)
 * - Pixmap drawing area (PixmapDraw)
 * - Color indicator
 */

#ifndef PixmapWnd_included
#define PixmapWnd_included

#include <qframe.h>
#include <qmainwindow.h>
#include <qstring.h>
#include <qscrbar.h>

#include "PixmapDraw.h"

class Pixmap;
class ProjectWnd;
class ColorFrame;
class QButtonGroup;
class QLabel;
class QPopupMenu;
class QToolButton;

#define SCRBAR_DIMENSION 15

class Previewer : public QFrame
{
  Q_OBJECT
public:
  Previewer( QWidget * );

  void SetPixmap( QPixmap * );

  void Resize();

private slots:
  void ScrBar( int );

private:
  virtual void drawContents( QPainter * );
  virtual void resizeEvent( QResizeEvent * );

private:
  QPixmap *contents;

  QScrollBar *horScr, *verScr;

  bool noPreview;
};

class PixmapWnd : public QMainWindow
{
    Q_OBJECT

public: // ctor, dtor
    PixmapWnd( Pixmap *, ProjectWnd *, const char * = NULL );
 
    ~PixmapWnd();

public: // methods
    PixmapDraw *GetPixmapDraw();


    void SetCaption( const QString&	captionStr );
    // Set the caption on the window

public slots:
    void MouseTracking( int, int, int, int );
    void ColorChanged( int, const QColor & );
    void ZoomChanged( int );

signals:
void Closed();

private: // methods
    void closeEvent( QCloseEvent * );
    // Override to prevent Qt from acting...

    void resizeEvent( QResizeEvent * );

private slots: // methods
    void Import();
    void Export();
    void Close();
    void SetOptions();
    void Rotate();
    void ModeChange( int );
    void PixelSize( int );
    void LineStyles( int );
    void UseGrid();
    void Select();
    void Deselect();
    void Clip();
    void Cut();
    void Copy();
    void CPAvailable();

private: // attributes
    PixmapDraw *itsPixmapDraw; 
    // Its drawing area

    Pixmap *pixmap;
    // Its master class

    int deselectEntry;
    int clipEntry;
    int cutEntry;
    int copyEntry;
    int pasteEntry;
    // Some menu entries from the edit menu which had to be disabled sometimes

    QButtonGroup* itsModeGroup;
    // Button group for holding the buttons for the drawing mode

    QLabel*	itsStatusZoom;
    QLabel*	itsStatusPSize;
    QLabel*	itsStatusPos;
    QLabel*	itsStatusSize;
    // identifiers for status bar regions

    int useGridMenu;
    // Identifier for the use grid menu entry

    ProjectWnd *theProjectWnd;
    // The application

    QPopupMenu *edit;
    QPopupMenu *tools;
    QPopupMenu *modes;
    QPopupMenu *sizes;
    QPopupMenu *styles;
    // Different drawing styles

    ColorFrame *itsColorArea;
    // Area for choosing colors, etc.

    Previewer *itsPreviewArea;
    // Area for showing pixmap previews in
};

inline PixmapDraw *PixmapWnd::GetPixmapDraw()
{
  return itsPixmapDraw;
}

inline void PixmapWnd::SetCaption
(
    const QString&	captionStr
)
{
    setCaption( "Qt Architect : " + captionStr );
    setIconText( caption() );
}


#endif
