/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ComboBoxPage.cpp,v 2.6 2000/04/30 17:09:39 jeff Exp $

*******************************************************************************/


#include <qcombo.h> 
#include <qstring.h>

#include "ComboBoxPage.h"

#include "widget/DlgWidget.h"
#include "widget/DlgComboBox.h"

#include "GuiTools.h"

#define Inherited ComboBoxPageData

ComboBoxPage::ComboBoxPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    const QComboBox* combo = (const QComboBox*)widget->GetWidget();
    const DlgComboBox* dlgCombo = (const DlgComboBox*)widget;
	
    connect( itsNewButton, SIGNAL(clicked()), SLOT(NewClicked()) );
    connect( itsChangeButton, SIGNAL(clicked()), SLOT(ChangeClicked()) );
    connect( itsDeleteButton, SIGNAL(clicked()), SLOT(DeleteClicked()) );
    connect( itsList, SIGNAL(highlighted(int)), 
	     SLOT(ListSelChanged(int)) );
    connect( itsNewLine, SIGNAL(textChanged(const QString&)),
	     SLOT(NewItemChanged(const QString&)) );
    connect( itsStyleGroup, SIGNAL(clicked(int)), 
	     SLOT(StyleChanged(int)) );

    GuiTools::TransferListFromCombo( combo, itsList );

    switch( dlgCombo->GetComboType() )
    {
	case DlgComboBox::CT_ReadOnly:
	default:
	{
	    isStyleRO->setChecked( TRUE );
	    break;
	}
	case DlgComboBox::CT_ReadWrite:
	{
	    isStyleRW->setChecked( TRUE );
	    break;
	}
	case DlgComboBox::CT_Motif1x:
	{
	    isStyleOld->setChecked( TRUE );
	    break;
	}
    }

    QString str;

    itsSizeLimit->setText( str.setNum( combo->sizeLimit() ) );
    itsMaxCount->setText( str.setNum( combo->maxCount() ) );
    itsPolicy->setCurrentItem( combo->insertionPolicy() );
    isAutoCompletion->setChecked( combo->autoCompletion() );
    isDuplicatesEnabled->setChecked( combo->duplicatesEnabled() );

    Update();
}


ComboBoxPage::~ComboBoxPage()
{
}


void ComboBoxPage::Apply( DlgWidget*	widget )
{
    QComboBox* combo = (QComboBox*)widget->GetWidget();
    DlgComboBox* dlgCombo = (DlgComboBox*)widget;
    
    GuiTools::TransferListToCombo( combo, itsList );

    DlgComboBox::ComboType type = DlgComboBox::CT_Motif1x;
    if( isStyleRO->isChecked() )
    {
	type = DlgComboBox::CT_ReadOnly;
    }
    else if( isStyleRW->isChecked() )
    {
	type = DlgComboBox::CT_ReadWrite;
    }
    
    dlgCombo->SetComboType( type );

    QString str = itsSizeLimit->text();
    combo->setSizeLimit( str.toInt() );

    str = itsMaxCount->text();
    combo->setMaxCount( str.toInt() );

    combo->setInsertionPolicy( (QComboBox::Policy)
			       itsPolicy->currentItem() );
    combo->setAutoCompletion( isAutoCompletion->isChecked() );
    combo->setDuplicatesEnabled( isDuplicatesEnabled->isChecked() );
}


void ComboBoxPage::NewClicked()
{
    const char* text = itsNewLine->text();
    if( text && text[0] )
    {
	itsList->insertItem( text );
    }
}


void ComboBoxPage::ChangeClicked()
{
    const char* text = itsNewLine->text();
    if( text && text[0] )
    {
	itsList->changeItem( text, itsList->currentItem() );
    }
}


void ComboBoxPage::DeleteClicked()
{
    int item = itsList->currentItem();
    if( item >= 0 )
    {
	itsList->removeItem( item );
	Update();
    }
}


void ComboBoxPage::ListSelChanged( int item )
{
    itsNewLine->setText( itsList->text( item ) );
    Update();
}


void ComboBoxPage::NewItemChanged( const QString& )
{
    Update();
}


void ComboBoxPage::StyleChanged( int )
{
    Update();
}


void ComboBoxPage::Update()
{
    const char* text = itsNewLine->text();
    bool newText = ( text && text[0] );
    bool itemSelected = (itsList->currentItem() != -1);

    itsNewButton->setEnabled( newText );
    itsChangeButton->setEnabled( newText & itemSelected );
    itsDeleteButton->setEnabled( itemSelected );

    bool enableLines = FALSE;
    bool enableInsert = FALSE;
    bool enableAutoCompletion = FALSE;
    bool enableEnableMultipleInsertion = FALSE;

    if( isStyleRO->isChecked() )
    {
	enableLines = TRUE;
    }
    else if( isStyleRW->isChecked() )
    {
	enableLines = TRUE;
	enableInsert = TRUE;
	enableAutoCompletion = TRUE;
        enableEnableMultipleInsertion = TRUE;
    }
    
    itsSizeLimitLabel->setEnabled( enableLines );
    itsSizeLimit->setEnabled( enableLines );
    itsPolicyLabel->setEnabled( enableInsert );
    itsPolicy->setEnabled( enableInsert );
    isAutoCompletion->setEnabled( enableAutoCompletion );
}


