/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LineEditPage.cpp,v 2.2 1999/10/16 01:48:58 jeff Exp $

*******************************************************************************/


#include <qlined.h>
#include <qstring.h>

#include "LineEditPage.h"

#include "widget/DlgWidget.h"

#define Inherited LineEditPageData

LineEditPage::LineEditPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    QLineEdit* edit = (QLineEdit*)widget->GetWidget();
    
    itsText->setText( edit->text() );

    switch( edit->echoMode() )
    {
	case QLineEdit::Normal:
	{
	    isEchoNormal->setChecked( TRUE );
	    break;
	}
	case QLineEdit::NoEcho:
	{
	    isEchoNone->setChecked( TRUE );
	    break;
	}
	case QLineEdit::Password:
	{
	    isEchoPassword->setChecked( TRUE );
	    break;
	}
    }

    QString str;
    str.setNum( edit->maxLength() );
    itsMaxLength->setText( str );

    isFrameShown->setChecked( edit->frame() );

    switch( edit->alignment() )
    {
        case Qt::AlignLeft:
        {
            isAlignLeft->setChecked( TRUE );
            break;
        }
        case Qt::AlignCenter:
        {
            isAlignCenter->setChecked( TRUE );
            break;
        }
        case Qt::AlignRight:
        {
            isAlignRight->setChecked( TRUE );
            break;
        }
    }
}


LineEditPage::~LineEditPage()
{
}


void LineEditPage::Apply( DlgWidget*	widget )
{
    QLineEdit* edit = (QLineEdit*)widget->GetWidget();
    
    edit->setText( itsText->text() );

    QLineEdit::EchoMode echo = QLineEdit::Normal;
    if( isEchoNone->isChecked() )
    {
	echo = QLineEdit::NoEcho;
    }
    else if( isEchoPassword->isChecked() )
    {
	echo = QLineEdit::Password;
    }
    edit->setEchoMode( echo );

    QString str = itsMaxLength->text();
    edit->setMaxLength( str.toInt() );

    edit->setFrame( isFrameShown->isChecked() );

    int alignment = 0;
    if( isAlignLeft->isChecked() )
    {
        alignment = Qt::AlignLeft;
    }
    else if( isAlignCenter->isChecked() )
    {
        alignment = Qt::AlignCenter;
    }
    else if( isAlignRight->isChecked() )
    {
        alignment = Qt::AlignRight;
    }
    edit->setAlignment( alignment );
}

