/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetColorPage.cpp,v 2.4 2000/04/30 17:09:43 jeff Exp $

*******************************************************************************/


#include <qcolordialog.h>
#include <qfontdialog.h>

#include "WidgetColorPage.h"

#include "widget/DlgWidget.h"

#include "GuiTools.h"

#define Inherited WidgetColorPageData

WidgetColorPage::WidgetColorPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name ),
    itsCurrentGroup( Active ),
    isCopy( FALSE ),
    areColorsChanged( FALSE ),
    isFontChanged( FALSE )
{
    itsPalette = widget->GetWidget()->palette();

    ChangeColors( itsPalette.active() );

    itsFont = widget->GetWidget()->font();

    UpdateFontInfo();

    connect( itsColorGroups, SIGNAL(activated(int)), SLOT(GroupChanged(int)) );
    connect( itsForeButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsBackButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsLightButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsDarkButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsMediumButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsTextButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsBaseButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsButtonButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsMidlightButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsShadowButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsHighlightButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsHighlightedTextButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsBrightTextButton, SIGNAL(clicked()), SLOT(ColorClicked()) );
    connect( itsButtonTextButton, SIGNAL(clicked()), SLOT(ColorClicked()) );

    connect( itsChooseButton, SIGNAL(clicked()), SLOT(FontClicked()) );
}


WidgetColorPage::~WidgetColorPage()
{
}


void WidgetColorPage::Apply( DlgWidget*	widget )
{
    if( areColorsChanged )
    {
	SaveCurrentGroup();
	widget->GetWidget()->setPalette( itsPalette );
    }

    if( isFontChanged )
    {
	widget->GetWidget()->setFont( itsFont );
    }
}


void WidgetColorPage::FontClicked()
{
    bool ok = FALSE;
    QFont font = QFontDialog::getFont( &ok, itsFont );
    
    if( ok )
    {
        itsFont = font;
        UpdateFontInfo();
        
        isFontChanged = TRUE;
    }
}


void WidgetColorPage::UpdateFontInfo()
{
    myFontLabel->setText( itsFont.rawName() );
}


void WidgetColorPage::SaveCurrentGroup( void )
{
    if( !isCopy )
    {
	itsPalette = itsPalette.copy();
	isCopy = TRUE;
    }

    QColorGroup group;
    group.setColor( QColorGroup::Foreground, 
                    itsForeButton->backgroundColor() );
    group.setColor( QColorGroup::Button, 
                    itsButtonButton->backgroundColor() );
    group.setColor( QColorGroup::Light, 
                    itsLightButton->backgroundColor() );
    group.setColor( QColorGroup::Midlight, 
                    itsMidlightButton->backgroundColor() );
    group.setColor( QColorGroup::Dark, 
                    itsDarkButton->backgroundColor() );
    group.setColor( QColorGroup::Mid, 
                    itsMediumButton->backgroundColor() );
    group.setColor( QColorGroup::Text, 
                    itsTextButton->backgroundColor() );
    group.setColor( QColorGroup::BrightText, 
                    itsBrightTextButton->backgroundColor() );
    group.setColor( QColorGroup::ButtonText, 
                    itsButtonTextButton->backgroundColor() );
    group.setColor( QColorGroup::Base, 
                    itsBaseButton->backgroundColor() );
    group.setColor( QColorGroup::Background, 
                    itsBackButton->backgroundColor() );
    group.setColor( QColorGroup::Shadow, 
                    itsShadowButton->backgroundColor() );
    group.setColor( QColorGroup::Highlight, 
                    itsHighlightButton->backgroundColor() );
    group.setColor( QColorGroup::HighlightedText, 
                    itsHighlightedTextButton->backgroundColor() );
    
    switch( itsCurrentGroup )
    {
	case Active:
	{
	    itsPalette.setActive( group );
	    break;
	}
	case Inactive:
	{
	    itsPalette.setInactive( group );
	    break;
	}
	case Disabled:
	{
	    itsPalette.setDisabled( group );
	    break;
	}
    }
}


void WidgetColorPage::ColorClicked()
{
    QWidget* widget = (QWidget*)sender();

    QColor color = QColorDialog::getColor( widget->backgroundColor() );
    if( color.isValid() )
    {
        areColorsChanged = TRUE;
        GuiTools::ChangeColor( widget, color );
    }
}


void WidgetColorPage::SetPaletteClicked()
{
    QColor color = QColorDialog::getColor( gray );
    if( color.isValid() )
    {
	areColorsChanged = TRUE;
	
	QPalette palette( color, color );
	itsPalette = palette;
	
	switch( itsCurrentGroup )
	{
	    case Active:
	    {
		ChangeColors( palette.active() );
		break;
	    }
	    case Inactive:
	    {
		ChangeColors( palette.inactive() );
		break;
	    }
	    case Disabled:
	    {
		ChangeColors( palette.disabled() );
		break;
	    }
	}
    }
}


void WidgetColorPage::GroupChanged( int group )
{
    SaveCurrentGroup();

    switch( group )
    {
	case Active:
	default:
	{
	    ChangeColors( itsPalette.active() );
	    break;
	}
	case Inactive:
	{
	    ChangeColors( itsPalette.inactive() );
	    break;
	}
	case Disabled:
	{
	    ChangeColors( itsPalette.disabled() );
	    break;
	}
    }

    itsCurrentGroup = group;
}


void WidgetColorPage::ChangeColors( const QColorGroup&	colorGroup )
{
    GuiTools::ChangeColor( itsForeButton, colorGroup.foreground() );
    GuiTools::ChangeColor( itsButtonButton, colorGroup.button() );
    GuiTools::ChangeColor( itsLightButton, colorGroup.light() );
    GuiTools::ChangeColor( itsDarkButton, colorGroup.dark() );
    GuiTools::ChangeColor( itsMediumButton, colorGroup.mid() );
    GuiTools::ChangeColor( itsTextButton, colorGroup.text() );
    GuiTools::ChangeColor( itsBaseButton, colorGroup.base() );
    GuiTools::ChangeColor( itsBackButton, colorGroup.background() );
    GuiTools::ChangeColor( itsMidlightButton, colorGroup.midlight() );
    GuiTools::ChangeColor( itsBrightTextButton, colorGroup.brightText() );
    GuiTools::ChangeColor( itsButtonTextButton, colorGroup.buttonText() );
    GuiTools::ChangeColor( itsShadowButton, colorGroup.shadow() );
    GuiTools::ChangeColor( itsHighlightButton, colorGroup.highlight() );
    GuiTools::ChangeColor( itsHighlightedTextButton, colorGroup.highlightedText() );
}


