/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgListBox.h,v 2.7 2000/04/30 17:09:48 jeff Exp $

*******************************************************************************/


#ifndef DlgListBox_included
#define DlgListBox_included

#include <qlistbox.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgScrollView.h"

class DlgListBox : public DlgScrollView
{

public:

    DlgListBox();
    // Constructor

    virtual ~DlgListBox();
    // Destructor
    

    //
    // Saving and Loading Methods
    //

    virtual void Save( QtArchGenerator & );
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair

    virtual void RestoreXML( const XMLValue & );
    // Restore a XMLValue


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
        const WidgetLayout&     widgetLayout
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals

    //
    // Utility methods
    //

    static QString SelectionModeToStr( QListBox::SelectionMode mode );
    static QListBox::SelectionMode StrToSelectionMode(
        const QString&  str 
        );
    // SelectionMode enum

    static QString LayoutModeToStr( QListBox::LayoutMode mode );
    static QListBox::LayoutMode StrToLayoutMode( const QString& str );
    // LayoutMode enum

protected: // Methods

    DlgListBox(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.


private: // Methods

    QListBox* GetListBox();
    // Get the Qt Widget as QListBox.

private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

};


inline QString DlgListBox::GetHeaderFilename() const
{
    return "<qlistbox.h>";
}

inline QString DlgListBox::GetWidgetClass() const
{
    return "QListBox";
}


inline QListBox* DlgListBox::GetListBox()
{
    return static_cast<QListBox*> (GetWidget());
}


#endif // DlgListBox_included
