/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgMenuBar.cpp,v 2.7 2000/05/04 03:20:16 jeff Exp $

*******************************************************************************/


#include "widget/DlgMenuBar.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgMenuBar::itsSignals;
int DlgMenuBar::itsNextWidgetNum = 1;

DlgMenuBar::DlgMenuBar() :
    Inherited( DlgWidget::DW_MenuBar )
{
}


DlgMenuBar::DlgMenuBar(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgMenuBar::~DlgMenuBar()
{
}


void DlgMenuBar::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );
}

void DlgMenuBar::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "MenuBar" );
  
  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgMenuBar::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // No known keys
    retVal = Inherited::RestoreKeyValue( key, value, FALSE );

    return retVal;
}

void DlgMenuBar::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "MenuBar" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetMenuBar() )
  {
    CreateQtWidget( 0 );
  }
  
  // We inherit from 
	Inherited::RestoreXML( map["Frame"] );

}


void DlgMenuBar::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );
}


void DlgMenuBar::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgMenuBar::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "activated(int)" );
	itsSignals.append( "highlighted(int)" );
    }

    return itsSignals;
}


void DlgMenuBar::CreateQtWidget(
    const char*	theName
    )
{
    QMenuBar* widget = 
	new QMenuBar( GetParent(),
		      CreateName( "MenuBar", 
				  theName, 
				  itsNextWidgetNum ) );

    SetQtWidget( widget );
}
